define("customPicaxeBasic_grove", [
  "blocklylib"
  ,"./blockly/generators/picaxebasic"
  ,"./blockly/msg"
   ,"./blockly/io"  
], function (BlocklyLib, generators, msg, io) {

  function hexToR(h) {return parseInt((cutHex(h)).substring(0,2),16)};
  function hexToG(h) {return parseInt((cutHex(h)).substring(2,4),16)};
  function hexToB(h) {return parseInt((cutHex(h)).substring(4,6),16)};
  function cutHex(h) {return (h.charAt(0)=="#") ? h.substring(1,7):h};
  function getPin2(pin1) {
    var pin = pin1.toUpperCase();
    switch (pin) {
      case "B.0":
        return "B.1"
        break;
      case "B.1":
        return "B.2"
        break;
      case "B.2":
        return "B.3"
        break;
      case "B.3":
        return "B.4"
        break;
      case "B.6":
        return "C.0"
        break;
      case "B.7":
        return "B.5"
        break;
      case "C.1":
        return "C.2"
        break;
      case "C.2":
        return "C.3"
        break;
      case "C.3":
        return "C.7"
        break;
      case "C.4":
        return "C.5"
        break;
      default:
        return pin
        break;
    };
  };

 var grove_symbol_header = ';\tTemporary word variables used by Grove modules.\n';
 grove_symbol_header += 'symbol grove_w1 = s_w1\n';
 grove_symbol_header += 'symbol grove_w2 = s_w3\n';
 grove_symbol_header += 'symbol grove_w3 = s_w4\n';
 grove_symbol_header += 'symbol grove_w4 = s_w6\n';
 grove_symbol_header += 'symbol grove_w5 = _h_w0\n';
 grove_symbol_header += 'symbol grove_w6 = _h_w1\n';
 grove_symbol_header += 'symbol grove_w7 = _h_w2\n';

 var x2part_header = '';
 x2part_header += '#ifdef _20X2 ;\n';
 x2part_header += '#define _X2part\n';
 x2part_header += '#endif\n';
 x2part_header += '#ifdef _28X2 ;\n';
 x2part_header += '#define _X2part\n';
 x2part_header += '#endif\n';
 x2part_header += '#ifdef _40X2 ;\n';
 x2part_header += '#define _X2part\n';
 x2part_header += '#endif\n';

 var uart_9600_header = '\t;Initialise serial baud rate\n';
 uart_9600_header += '#ifdef _X2part\n';
 uart_9600_header += '\thsersetup B9600_8, %001\n';
 uart_9600_header += '#else\n';
 uart_9600_header += '\thsersetup B9600_4, %000\n';
 uart_9600_header += '#endif\n';

 //v1 3414
  var grove_i2c_colour_header = '\t;Initialise I2C Colour module\n';
  grove_i2c_colour_header += "\tHI2cSetup I2CMASTER, $72, I2CSLOW, I2CBYTE\n";
  grove_i2c_colour_header += "\tHI2cOut %10000001, (%00000000) ; Free-Run, 12ms\n";
  grove_i2c_colour_header += "\tHI2cOut %10000111, (%00110000) ; 64x gain, divide by 1 pre-scaler\n";

  //v2 34725
  var grove_i2c_colour_v2_header  = '\t;Initialise I2C Colour module\n';
  grove_i2c_colour_v2_header += "\tHI2cSetup I2CMASTER, $52, I2CSLOW, I2CBYTE\n";
  grove_i2c_colour_v2_header += "\tHI2cOut %10000001, (0xF6)      ; 24ms\n";
  grove_i2c_colour_v2_header += "\tHI2cOut %10001111, (%00000010) ; 16x gain\n";
  grove_i2c_colour_v2_header += "\tHI2cOut %10000000, (%00000001) ; PON enable\n";
  grove_i2c_colour_v2_header += "\tpause 3\n";
  grove_i2c_colour_v2_header += "\tHI2cOut %10000000, (%00000011) ; AEN enable\n";
  
  
generators['grove_i2c_colour_sensor'] = function(block) {
  if (!generators.definitions_['grove_symbol_header']) generators.definitions_['grove_symbol_header'] = grove_symbol_header;
  var varG = generators.variableDB_.getName(block.getFieldValue('VARG'), BlocklyLib.Variables.NAME_TYPE);
  var varR = generators.variableDB_.getName(block.getFieldValue('VARR'), BlocklyLib.Variables.NAME_TYPE);
  var varB = generators.variableDB_.getName(block.getFieldValue('VARB'), BlocklyLib.Variables.NAME_TYPE);  
  var varW = generators.variableDB_.getName(block.getFieldValue('VARW'), BlocklyLib.Variables.NAME_TYPE);
  var type = block.getFieldValue('TYPE');
  var code = "";
  
  if (type == 20) { //v2.0
    if (!generators.definitions_['grove_i2c_colour_sensor']) generators.definitions_['grove_i2c_colour_sensor'] = grove_i2c_colour_v2_header;     
    code += "\tHI2cSetup I2CMASTER, $52, I2CSLOW, I2CBYTE\n";
    code += "\tHI2cOut %10000000, (%00000011)\n";
    code += "\tHI2cIn  0x14, (" + varW + ", " + varW + ")\n";
    code += "\tHI2cIn  0x16, (" + varR + ", " + varR + ")\n";
    code += "\tHI2cIn  0x18, (" + varG + ", " + varG + ")\n";
    code += "\tHI2cIn  0x1A, (" + varB + ", " + varB + ")\n";
    code += "\tpause 25\n";   
  } else { //v1.2
    if (!generators.definitions_['grove_i2c_colour_sensor']) generators.definitions_['grove_i2c_colour_sensor'] = grove_i2c_colour_header;
    code += "\tHI2cSetup I2CMASTER, $72, I2CSLOW, I2CBYTE\n";
    code += "\tHI2cOut %10000000, (%00000011)\n";
    code += "\tpause 20\n";
    code += "\tHI2cIn  %11010000, (" + varG + ", " + varG + " , " + varR + ", " + varR + " , " + varB + ", " + varB + ", " + varW + "," + varW +")\n";
    }    
    
  return generators.wrapOutput(block, code);
};

generators['grove_rtc'] = function(block) {
  var var1 = generators.variableDB_.getName(block.getFieldValue('VAR1'), BlocklyLib.Variables.NAME_TYPE);
  var var2 = generators.variableDB_.getName(block.getFieldValue('VAR2'), BlocklyLib.Variables.NAME_TYPE);
  var var3 = generators.variableDB_.getName(block.getFieldValue('VAR3'), BlocklyLib.Variables.NAME_TYPE);  
  var type = block.getFieldValue('TYPE');
  var code = "\tHI2cSetup I2CMASTER, $D0, I2CSLOW, I2CBYTE\n";
  if (type == "Time") { 
    code += "\tHI2cIn  0x00, (" + var3 + ", " + var2 + ", " + var1 + ")\n";
  } else { 
    code += "\tHI2cIn  0x04, (" + var1 + ", " + var2 + ", " + var3 + ")\n";  
  }
  return generators.wrapOutput(block, code);
};

  var grove_nunchuck_header = '\t;Initialise nunchuck module\n';
  grove_nunchuck_header += "\tpause 10\n\tHI2cSetup I2CMASTER, $A4, I2CSLOW, I2CBYTE\n";
  grove_nunchuck_header += "\tHI2cOut 0x40, (0x00)\n";
  grove_nunchuck_header += "\tpause 1\n";

generators['grove_nunchuck'] = function(block) {
  if (!generators.definitions_['grove_nunchuck']) generators.definitions_['grove_nunchuck'] = grove_nunchuck_header;     
  var var1 = generators.variableDB_.getName(block.getFieldValue('VAR1'), BlocklyLib.Variables.NAME_TYPE);
  var var2 = generators.variableDB_.getName(block.getFieldValue('VAR2'), BlocklyLib.Variables.NAME_TYPE);
  var var3 = generators.variableDB_.getName(block.getFieldValue('VAR3'), BlocklyLib.Variables.NAME_TYPE);  
  var code = "\tHI2cSetup I2CMASTER, $A4, I2CSLOW, I2CBYTE\n";
  code += "\thi2cout (0)\n";
  code += "\thi2cin ("+var1+","+var2+","+var3+","+var3+","+var3+","+var3+")\n";
  code += "\t"+var1+" = "+var1+" ^ $17 + $17\n";
  code += "\t"+var2+" = "+var2+" ^ $17 + $17\n";
  code += "\t"+var3+" = "+var3+" ^ $17 + $17 & 3\n";
  code += "\tlookup "+var3+",(3,1,2,0),"+var3+"\n";
  return generators.wrapOutput(block, code);
};

  var grove_3axis_header_15 = '\t;Initialise 3axis module\n';
  grove_3axis_header_15 += "\tHI2cSetup I2CMASTER, $98, I2CSLOW, I2CBYTE\n";
  grove_3axis_header_15 += "\tHI2cOut 0x07, (0x00)\n";
  grove_3axis_header_15 += "\tHI2cOut 0x08, (0x02)\n";
  grove_3axis_header_15 += "\tHI2cOut 0x07, (0x01)\n";
  
  var grove_3axis_header_16 = '\t;Initialise 3axis module\n';
  grove_3axis_header_16 += "\tHI2cSetup I2CMASTER, $A6, I2CSLOW, I2CBYTE\n";
  grove_3axis_header_16 += "\tHI2cOut 0x2D, (0x00)\n";
  grove_3axis_header_16 += "\tHI2cOut 0x31, (%00001011)\n"; //13 bit at 16g
  //grove_3axis_header_16 += "\tHI2cOut 0x2D, (0x10)\n";
  grove_3axis_header_16 += "\tHI2cOut 0x2D, (0x08)\n";
  
  var grove_3axis_header_400 = '\t;Initialise 3axis module\n';
  grove_3axis_header_400 += "\tHI2cSetup I2CMASTER, $30, I2CSLOW, I2CBYTE\n";
  grove_3axis_header_400 += "\thi2cout 0x20, (%00101111)\n";
  grove_3axis_header_400 += "\thi2cout 0x23, (%00110000)\n";
  
generators['grove_3axis_digital_acc'] = function(block) {
  var var1 = generators.variableDB_.getName(block.getFieldValue('VAR1'), BlocklyLib.Variables.NAME_TYPE);
  var var2 = generators.variableDB_.getName(block.getFieldValue('VAR2'), BlocklyLib.Variables.NAME_TYPE);
  var var3 = generators.variableDB_.getName(block.getFieldValue('VAR3'), BlocklyLib.Variables.NAME_TYPE);  
  var type = block.getFieldValue('TYPE');
  var code = "";
  
  if (type == 15) { //1.5g
    if (!generators.definitions_['grove_3axis_15']) generators.definitions_['grove_3axis_15'] = grove_3axis_header_15;
    code += "\tHI2cSetup I2CMASTER, $98, I2CSLOW, I2CBYTE\n";
    code += "\tdo\n"
    code += "\t\thi2cin 0,("+var1+","+var2+","+var3+")\n";
    code += "\tloop while "+var1+" > 63\n"
  } else if (type == 16) { //16g
    if (!generators.definitions_['grove_symbol_header']) generators.definitions_['grove_symbol_header'] = grove_symbol_header;
    if (!generators.definitions_['grove_3axis_16']) generators.definitions_['grove_3axis_16'] = grove_3axis_header_16;     
    code += "\tHI2cSetup I2CMASTER, $A6, I2CSLOW, I2CBYTE\n";
    code += "\thi2cin 0x28,(grove_w1,"+var1+",grove_w2,"+var2+",grove_w3,"+var3+")\n";
    code += "\t"+var1+" = "+var1+"*256 | grove_w1\n";
    code += "\t"+var2+" = "+var2+"*256 | grove_w2\n";
    code += "\t"+var3+" = "+var3+"*256 | grove_w3\n";
  } else if (type == 400) { //400g
    if (!generators.definitions_['grove_symbol_header']) generators.definitions_['grove_symbol_header'] = grove_symbol_header;
    if (!generators.definitions_['grove_3axis_400']) generators.definitions_['grove_3axis_400'] = grove_3axis_header_400;     
    code += "\tHI2cSetup I2CMASTER, $30, I2CSLOW, I2CBYTE\n";
    code += "\thi2cin 0x28,(grove_w1,"+var1+",grove_w2,"+var2+",grove_w3,"+var3+")\n";
    code += "\t"+var1+" = "+var1+"*256 | grove_w1\n";
    code += "\t"+var2+" = "+var2+"*256 | grove_w2\n";
    code += "\t"+var3+" = "+var3+"*256 | grove_w3\n";
  }

  return generators.wrapOutput(block, code);
};

generators['grove_3axis_digital_acc_int'] = function(block) {
  var varS = generators.variableDB_.getName(block.getFieldValue('VARS'), BlocklyLib.Variables.NAME_TYPE);
  var varI = generators.variableDB_.getName(block.getFieldValue('VARI'), BlocklyLib.Variables.NAME_TYPE);
  //var varR = generators.variableDB_.getName(block.getFieldValue('VARR'), BlocklyLib.Variables.NAME_TYPE);
  var cmd = block.getFieldValue('COMMAND');
  var type = block.getFieldValue('TYPE');
  var code = "";
  
  if (type == 15) { //1.5g
    if (!generators.definitions_['grove_3axis_15']) generators.definitions_['grove_3axis_15'] = grove_3axis_header_15;
    var addr = '0';
    if (cmd == "Y") addr = '1'; 
    else if (cmd == "Z") addr = '2'; 
    code += "\tHI2cSetup I2CMASTER, $98, I2CSLOW, I2CBYTE\n";
    code += "\tdo\n"
    code += "\t\thi2cin "+addr+",("+varI+")\n";
    code += "\tloop while "+varI+" > 63\n"
    code += "\tif " + varI+" > 31 then\n"
    code += '\t\t' + varS + ' = "-"\n';
    code += "\t\t" + varI +" = "+varI + "^ 0x1F & 0x1F + 1\n";
    code += "\telse\n"
    code += '\t\t' + varS + ' = "+"\n'; 
    code += "\tend if\n"
  } else if (type == 16) { //16g
    if (!generators.definitions_['grove_symbol_header']) generators.definitions_['grove_symbol_header'] = grove_symbol_header;
    if (!generators.definitions_['grove_3axis_16']) generators.definitions_['grove_3axis_16'] = grove_3axis_header_16;     
    var addr = '0x28';
    if (cmd == "Y") addr = '0x2A';
    else if (cmd == "Z") addr = '0x2C';
    code += "\tHI2cSetup I2CMASTER, $A6, I2CSLOW, I2CBYTE\n";
    code += "\thi2cin "+addr+",(grove_w1,"+varI+")\n";
    code += "\t"+varI+" = "+varI+"*256 | grove_w1\n";
    code += "\tif " + varI+" > 0x0FFF then\n"
    code += '\t\t' + varS + ' = "-"\n';
    code += "\t\t" + varI +" = "+varI + "^ 0x0FFF & 0x0FFF + 1\n";
    code += "\telse\n"
    code += '\t\t' + varS + ' = "+"\n'; 
    code += "\tend if\n"
  } else if (type == 400) { //400g
    if (!generators.definitions_['grove_symbol_header']) generators.definitions_['grove_symbol_header'] = grove_symbol_header;
    if (!generators.definitions_['grove_3axis_400']) generators.definitions_['grove_3axis_400'] = grove_3axis_header_400;     
    var addr = '0x28';
    if (cmd == "Y") addr = '0x2A';
    else if (cmd == "Z") addr = '0x2C';
    code += "\tHI2cSetup I2CMASTER, $30, I2CSLOW, I2CBYTE\n";
    code += "\thi2cin "+addr+",(grove_w1,"+varI+")\n";
    code += "\t"+varI+" = "+varI+"*256 | grove_w1\n";
    code += "\tif " + varI+" > 0x7FFF then\n"
    code += '\t\t' + varS + ' = "-"\n';
    code += "\t\t" + varI +" = -"+varI + "\n";
    code += "\telse\n"
    code += '\t\t' + varS + ' = "+"\n'; 
    code += "\tend if\n"
    }

  return generators.wrapOutput(block, code);
};

  var grove_3axis_compass_header_10 = '\t;Initialise 3axis compass module\n';
  grove_3axis_compass_header_10 += "\tHI2cSetup I2CMASTER, $3C, I2CSLOW, I2CBYTE\n";
  grove_3axis_compass_header_10 += "\tHI2cOut 0x02, (0x00)\n";
  
  var grove_3axis_compass_header_20 = '\t;Initialise 3axis compass module\n';
  grove_3axis_compass_header_20 += "\tHI2cSetup I2CMASTER, $26, I2CSLOW, I2CBYTE\n";
  grove_3axis_compass_header_20 += "\tHI2cOut 0x4B, (0x01)\n";
  grove_3axis_compass_header_20 += "\tHI2cOut 0x4C, (0x00)\n"; 
  
generators['grove_3axis_digital_compass'] = function(block) {
  var var1 = generators.variableDB_.getName(block.getFieldValue('VAR1'), BlocklyLib.Variables.NAME_TYPE);
  var var2 = generators.variableDB_.getName(block.getFieldValue('VAR2'), BlocklyLib.Variables.NAME_TYPE);
  var var3 = generators.variableDB_.getName(block.getFieldValue('VAR3'), BlocklyLib.Variables.NAME_TYPE);  
  var type = block.getFieldValue('TYPE');
  var code = "";
  
  if (type == 10) { 
    if (!generators.definitions_['grove_3axis_compass_10']) generators.definitions_['grove_3axis_compass_10'] = grove_3axis_compass_header_10;
    code += "\tHI2cSetup I2CMASTER, $3C, I2CSLOW, I2CBYTE\n";
    code += "\thi2cin 0x03,(grove_w1,"+var1+",grove_w2,"+var2+",grove_w3,"+var3+")\n";
    code += "\t"+var1+" = "+var1+"*256 | grove_w1\n";
    code += "\t"+var2+" = "+var2+"*256 | grove_w2\n";
    code += "\t"+var3+" = "+var3+"*256 | grove_w3\n";
  } else if (type == 20) { 
    if (!generators.definitions_['grove_symbol_header']) generators.definitions_['grove_symbol_header'] = grove_symbol_header;
    if (!generators.definitions_['grove_3axis_compass_20']) generators.definitions_['grove_3axis_compass_20'] = grove_3axis_compass_header_20;     
    code += "\tHI2cSetup I2CMASTER, $26, I2CSLOW, I2CBYTE\n";
    code += "\thi2cin 0x42,(grove_w1,"+var1+",grove_w2,"+var2+",grove_w3,"+var3+")\n";
    code += "\t"+var1+" = "+var1+"*256 | grove_w1 /8\n";
    code += "\t"+var2+" = "+var2+"*256 | grove_w2 /8\n";
    code += "\t"+var3+" = "+var3+"*256 | grove_w3 /2\n";
  }
  return generators.wrapOutput(block, code);
};

generators['grove_3axis_digital_compass_int'] = function(block) {
  var varS = generators.variableDB_.getName(block.getFieldValue('VARS'), BlocklyLib.Variables.NAME_TYPE);
  var varI = generators.variableDB_.getName(block.getFieldValue('VARI'), BlocklyLib.Variables.NAME_TYPE);
  //var varR = generators.variableDB_.getName(block.getFieldValue('VARR'), BlocklyLib.Variables.NAME_TYPE);
  var cmd = block.getFieldValue('COMMAND');
  var type = block.getFieldValue('TYPE');
  var code = "";
  
  if (type == 10) { 
    if (!generators.definitions_['grove_3axis_compass_10']) generators.definitions_['grove_3axis_compass_10'] = grove_3axis_compass_header_10;
    var addr = '0x03';
    if (cmd == "Y") addr = '0x05'; 
    else if (cmd == "Z") addr = '0x07'; 
    code += "\tHI2cSetup I2CMASTER, $A6, I2CSLOW, I2CBYTE\n";
    code += "\thi2cin "+addr+",(grove_w1,"+varI+")\n";
    code += "\t"+varI+" = "+varI+"*256 | grove_w1\n";
    code += "\tif " + varI+" > 0x7FFF then\n"
    code += '\t\t' + varS + ' = "-"\n';
    code += "\t\t" + varI +" = -"+varI + "\n";
    code += "\telse\n"
    code += '\t\t' + varS + ' = "+"\n'; 
    code += "\tend if\n"
  } else if (type == 20) { 
    if (!generators.definitions_['grove_symbol_header']) generators.definitions_['grove_symbol_header'] = grove_symbol_header;
    if (!generators.definitions_['grove_3axis_compass_20']) generators.definitions_['grove_3axis_compass_20'] = grove_3axis_compass_header_20;     
    var addr = '0x42';
    if (cmd == "Y") addr = '0x44';
    else if (cmd == "Z") addr = '0x46';
    code += "\tHI2cSetup I2CMASTER, $26, I2CSLOW, I2CBYTE\n";
    code += "\thi2cin "+addr+",(grove_w1,"+varI+")\n";
    code += "\t"+varI+" = "+varI+"*256 | grove_w1\n";
    code += "\tif " + varI+" > 0x7FFF then\n"
    code += '\t\t' + varS + ' = "-"\n';
    code += "\t\t" + varI +" = -"+varI + "\n";
    code += "\telse\n"
    code += '\t\t' + varS + ' = "+"\n'; 
    code += "\tend if\n"
    if (cmd == "Z") code += "\t"+varI+" = "+varI+"/2\n";
    else code += "\t"+varI+" = "+varI+"/8\n";
  }
  return generators.wrapOutput(block, code);
};


  //http://www.hoperf.com/upload/sensor/HP206C_DataSheet_EN_V2.0.pdf
  var grove_barometer_high_header = '\t;Initialise barometer module\n';
  grove_barometer_high_header += "symbol grove_barometer_int = grove_w1\n";
  grove_barometer_high_header += "symbol grove_barometer_rem = grove_w2\n";
  grove_barometer_high_header += "symbol grove_barometer_sign = grove_w3\n";
  grove_barometer_high_header += "symbol grove_barometer_cmd = grove_w4\n";
  grove_barometer_high_header += "\thi2csetup i2cmaster, 0xEC, i2cslow, i2cbyte\n";
  grove_barometer_high_header += "\thi2cout (0x06)\n";

  var grove_barometer_high_gosub = "grove_barometer_read:\n";
  grove_barometer_high_gosub += "\thi2csetup i2cmaster, 0xEC, i2cslow, i2cbyte\n";
  grove_barometer_high_gosub += "\thi2cout (0x48)\n";
  grove_barometer_high_gosub += "\tpause 25\n";
  grove_barometer_high_gosub += "\thi2cin grove_barometer_cmd, (grove_w5, grove_w6, grove_w7)\n";
  grove_barometer_high_gosub += "\tgrove_w6 = grove_w6 * 256 + grove_w7\n";
  grove_barometer_high_gosub += "\tif grove_w5 < 0x80 then\n";
  grove_barometer_high_gosub += "\t\tgrove_w5 = grove_w5 & 0x07\n";
  grove_barometer_high_gosub += '\t\tgrove_barometer_sign = "+"\n';
  grove_barometer_high_gosub += "\telse\n";
  grove_barometer_high_gosub += "\t\tgrove_w5 = grove_w5 ^ 0x07 & 0x07 \n";
  grove_barometer_high_gosub += "\t\tgrove_w6 = -grove_w6 \n";
  grove_barometer_high_gosub += "\t\tgrove_w5 = grove_w6 max 1 ^ 1 + grove_w5\n";
  grove_barometer_high_gosub += '\t\tgrove_barometer_sign = "-"\n';
  grove_barometer_high_gosub += "\tend if\n";
  grove_barometer_high_gosub += "\t\tgrove_w7 = grove_w6 // 200\n";
  grove_barometer_high_gosub += "\t\tgrove_w7 = grove_w5 * 536 + grove_w7 // 200\n";
  grove_barometer_high_gosub += "\t\tgrove_w6 = grove_w6 / 8\n";
  grove_barometer_high_gosub += "\t\tgrove_w6 = grove_w5 * $2000 + grove_w6 / 25 * 2\n";
  grove_barometer_high_gosub += "\t\tgrove_barometer_int = grove_w7 / 100 + grove_w6\n";
  grove_barometer_high_gosub += "\t\tgrove_barometer_rem = grove_w7 // 100\n";
  grove_barometer_high_gosub += "\tif grove_w5 = 8 then\n";
  grove_barometer_high_gosub += "\t\tgrove_barometer_int = 5242\n";
  grove_barometer_high_gosub += "\tend if\n";
  grove_barometer_high_gosub += "\treturn\n";
  
generators['grove_barometer_high_accuracy'] = function (block) {
  if (!generators.definitions_['grove_symbol_header']) generators.definitions_['grove_symbol_header'] = grove_symbol_header;
  if (!generators.definitions_['grove_barometer_high']) generators.definitions_['grove_barometer_high'] = grove_barometer_high_header;
  if (!generators.gosubs_['grove_barometer_high']) generators.gosubs_['grove_barometer_high'] = grove_barometer_high_gosub;
  var varS = generators.variableDB_.getName(block.getFieldValue('VARS'), BlocklyLib.Variables.NAME_TYPE);
  var varI = generators.variableDB_.getName(block.getFieldValue('VARI'), BlocklyLib.Variables.NAME_TYPE);
  var varR = generators.variableDB_.getName(block.getFieldValue('VARR'), BlocklyLib.Variables.NAME_TYPE);
  var cmd = block.getFieldValue('COMMAND');
  
  var code = "\tgrove_barometer_cmd = " + cmd + "\n";
  code += "\tgosub grove_barometer_read\n";
  code += "\t" + varS + " = grove_barometer_sign\n";
  code += "\t" + varI + " = grove_barometer_int\n";
  code += "\t" + varR + " = grove_barometer_rem\n";
  return generators.wrapOutput(block, code);
};

generators['grove_temperature_humidity'] = function(block) {
  var varT = generators.variableDB_.getName(block.getFieldValue('VART'), BlocklyLib.Variables.NAME_TYPE);
  var varH = generators.variableDB_.getName(block.getFieldValue('VARH'), BlocklyLib.Variables.NAME_TYPE);
  var code = "#ERROR - device not supported\n";
  return generators.wrapOutput(block, code);
};

  var lcd_header = '\t;Initialise Serial LCD\n'
  lcd_header += '\thserout 0,(0x9f, 0x82)\n';
  lcd_header += '\tpause 1\n';
  lcd_header += '\thserout 0,(0x9f, 0x83)\n';
  lcd_header += '\tpause 1\n';
  lcd_header += '\thserout 0,($a3)\n';

generators['grove_serial_lcd_print'] = function(block) {
  if (!generators.definitions_['grove_define_x2part']) generators.definitions_['grove_define_x2part'] = x2part_header;
  if (!generators.definitions_['grove_define_uart_9600']) generators.definitions_['grove_define_uart_9600'] = uart_9600_header;
  if (!generators.definitions_['grove_serial_lcd_init']) generators.definitions_['grove_serial_lcd_init'] = lcd_header;
  var ascii = (block.getFieldValue('ASCII') == 'TRUE');
  var value = generators.valueToCode(block, 'VALUE', generators.ORDER_ATOMIC);
  if ( value[0] != '"' && ascii) {
    value = "#" + value;
    value = value.replace(/, /g, ", #"); 
  } else if ( value[0] != '"' ) {
    value = value;
  } else {
    value = generators.convertAnsiiToLcdChars(value);
  }
  var code = "\thserout 0,(0xFE, " + value + " )\n";
  return generators.wrapOutput(block, code);
};

generators['grove_serial_lcd_command'] = function(block) {
  if (!generators.definitions_['grove_define_x2part']) generators.definitions_['grove_define_x2part'] = x2part_header;    
  if (!generators.definitions_['grove_define_uart_9600']) generators.definitions_['grove_define_uart_9600'] = uart_9600_header;
  if (!generators.definitions_['grove_serial_lcd_init']) generators.definitions_['grove_serial_lcd_init'] = lcd_header;
  var codetag = block.getFieldValue('CODETAG');
  var code = "\thserout 0,( "+codetag + " )\n";
  if ( codetag == "0x9F, 0x01" ) {
    code += "\tpause 10\n";
  }
  return generators.wrapOutput(block, code);
};

  //Grove - OLED Display 0.96"       128x 64   104030008  Mode 0
  //Grove - OLED Display 1.12" V2    128x128   101020452  Mode 1
  //Grove - OLED Display 1.12"        96x 96   104030011  Mode 2

  var grove_oled_header = '\t;Initialise OLED module\n'
  grove_oled_header += 'symbol grove_oled_row = grove_w1\n';
  grove_oled_header += 'symbol grove_oled_col = grove_w2\n';
  grove_oled_header += 'symbol grove_oled_chr = grove_w3\n';
  grove_oled_header += 'symbol grove_oled_temp = grove_w4\n';
  grove_oled_header += '\tgosub grove_oled_init\n';

  var grove_oled_gosub = "grove_oled_init:\n";
  grove_oled_gosub += "\tHI2cSetup I2CMASTER, $78, I2CSLOW, I2CBYTE\n";
  grove_oled_gosub += "\tHI2cOut $80, (0xAE)\n";
  grove_oled_gosub += "\tHI2cOut $80, (0xA4)\n";
  grove_oled_gosub += "\tHI2cOut $80, (0x20)\n";
  grove_oled_gosub += "\tHI2cOut $80, (0x02)\n";

  grove_oled_gosub += "\tHI2cOut $80, (0xA1)\n";
  grove_oled_gosub += "\tHI2cOut $80, (0xC8)\n";
  grove_oled_gosub += "\tHI2cOut $80, (0xAF)\n";
  grove_oled_gosub += "\tpause 100\n\n";
  grove_oled_gosub += "grove_oled_clearScreen:\n";
  grove_oled_gosub += "\tHI2cSetup I2CMASTER, $78, I2CSLOW, I2CBYTE\n";
  grove_oled_gosub += "\tFor grove_oled_row = $B0 To $B7\n";
  grove_oled_gosub += "\t\tFor grove_oled_col = 0 To 127 Step 8\n";
  grove_oled_gosub += "\t\t\tHI2cOut $80, (grove_oled_row)\n";
  grove_oled_gosub += "\t\t\tgrove_oled_temp = grove_oled_col & 0x0F\n";
  grove_oled_gosub += "\t\t\tHI2cOut $80, (grove_oled_temp)\n";
  grove_oled_gosub += "\t\t\tgrove_oled_temp = grove_oled_col / 16 + 0x10\n";
  grove_oled_gosub += "\t\t\tHI2cOut $80, (grove_oled_temp)\n";
  grove_oled_gosub += "\t\t\tHI2cOut $40,(0,0,0,0,0,0,0,0)\n";
  grove_oled_gosub += "\t\tNext\n";
  grove_oled_gosub += "\tNext\n";
  grove_oled_gosub += "\tgrove_oled_row = 0\n";
  grove_oled_gosub += "\tgrove_oled_col = 0\n";
  grove_oled_gosub += "\tReturn\n\n";

  grove_oled_gosub += "grove_oled_putChar:\n";
  grove_oled_gosub += "\tHI2cSetup I2CMASTER, $78, I2CSLOW, I2CBYTE\n";
  grove_oled_gosub += "\tgrove_w7 = 0xB0 + grove_oled_row\n";
  grove_oled_gosub += "\tHI2cOut $80, (grove_w7)\n";
  grove_oled_gosub += "\tgrove_w7 = 8 * grove_oled_col & 0x0F\n";
  grove_oled_gosub += "\tHI2cOut $80, (grove_w7)\n";
  grove_oled_gosub += "\tgrove_w7 = 8 * grove_oled_col  / 16 + 0x10\n";
  grove_oled_gosub += "\tHI2cOut $80, (grove_w7)\n";

  grove_oled_gosub += "#ifndef _X2part\n";  
    grove_oled_gosub += "\tIf grove_oled_chr < $1A Or grove_oled_chr > $AF Then\n";
    grove_oled_gosub += '\t\tgrove_oled_chr = " "\n';
    grove_oled_gosub += "\tEnd If\n";  
  grove_oled_gosub += "#else\n";  
    grove_oled_gosub += "\tIf grove_oled_chr < $1A Or grove_oled_chr > $7F Then\n";
    grove_oled_gosub += '\t\tgrove_oled_chr = " "\n';
    grove_oled_gosub += "\tEnd If\n";  
  grove_oled_gosub += "#endif\n";  

  grove_oled_gosub += "\tIf grove_oled_chr >= $4D Then\n";
  grove_oled_gosub += "\t\tgrove_w6 = grove_oled_chr - $4D * 5\n";
  grove_oled_gosub += "\tElse\n";
  grove_oled_gosub += "\t\tgrove_w6 = grove_oled_chr - $1A * 5 + 1\n";
  grove_oled_gosub += "\tEnd If\n";

  grove_oled_gosub += "\tFor grove_w5 = 0 To 7\n";
  grove_oled_gosub += "\t\tIf grove_w5 >= 5 then\n";
  grove_oled_gosub += "\t\t\tgrove_w7 = 0\n";
  grove_oled_gosub += "\t\telse\n";
  grove_oled_gosub += "\t\t\tIf grove_oled_chr >= $4D Then\n";
  grove_oled_gosub += "\t\t\t\tReadTable grove_w6,grove_w7\n";
  grove_oled_gosub += "\t\t\tElse\n";
  grove_oled_gosub += "\t\t\t\tRead grove_w6,grove_w7\n";
  grove_oled_gosub += "\t\t\tEnd If\n";
  grove_oled_gosub += "\t\t\tgrove_w6 = grove_w6 + 1\n";
  grove_oled_gosub += "\t\tend if\n";
  grove_oled_gosub += "\t\tHI2cOut $40, (grove_w7)\n";
  grove_oled_gosub += "\tNext\n";

  grove_oled_gosub += "\tgrove_oled_col = grove_oled_col + 1 & $0F\n";
  grove_oled_gosub += "\tIf grove_oled_col = 0 Then\n";
  grove_oled_gosub += "\t\tgrove_oled_row = grove_oled_row + 1 // 8\n";
  grove_oled_gosub += "\tEnd If\n";
  grove_oled_gosub += "\tReturn\n\n";

  grove_oled_gosub += "grove_oled_putValue:\n";
  grove_oled_gosub += "\tif grove_oled_temp > 999 then\n";
  grove_oled_gosub += '\t\tgrove_oled_chr = grove_oled_temp / 10000 + "0"\n';
  grove_oled_gosub += "\t\tgosub grove_oled_putChar\n";
  grove_oled_gosub += '\t\tgrove_oled_chr = grove_oled_temp / 1000 // 10 + "0"\n';
  grove_oled_gosub += "\t\tgosub grove_oled_putChar\n";
  grove_oled_gosub += "\tend if\n";
  grove_oled_gosub += '\tgrove_oled_chr = grove_oled_temp / 100 // 10 + "0"\n';
  grove_oled_gosub += "\tgosub grove_oled_putChar\n";
  grove_oled_gosub += '\tgrove_oled_chr = grove_oled_temp / 10 // 10 + "0"\n';
  grove_oled_gosub += "\tgosub grove_oled_putChar\n";
  grove_oled_gosub += '\tgrove_oled_chr = grove_oled_temp // 10 + "0"\n';
  grove_oled_gosub += "\tgosub grove_oled_putChar\n";
  grove_oled_gosub += "\treturn\n\n";

  var grove_oled_gosub1 = "grove_oled_init:\n";
  grove_oled_gosub1 += "\tHI2cSetup I2CMASTER, $78, I2CSLOW, I2CBYTE\n"; 
  grove_oled_gosub1 += "\tHI2cOut $80, (0xFD)\n";
  grove_oled_gosub1 += "\tHI2cOut $80, (0x12)\n";
  grove_oled_gosub1 += "\tHI2cOut $80, (0xAE)\n";
  grove_oled_gosub1 += "\tHI2cOut $80, (0xA8)\n";
  grove_oled_gosub1 += "\tHI2cOut $80, (0x5F)\n";
  grove_oled_gosub1 += "\tHI2cOut $80, (0xA1)\n";
  grove_oled_gosub1 += "\tHI2cOut $80, (0x00)\n";
  grove_oled_gosub1 += "\tHI2cOut $80, (0xA2)\n";
  grove_oled_gosub1 += "\tHI2cOut $80, (0x60)\n";
  grove_oled_gosub1 += "\tHI2cOut $80, (0xA0)\n";  
  grove_oled_gosub1 += "\tHI2cOut $80, (0x46)\n";
  grove_oled_gosub1 += "\tHI2cOut $80, (0xAB)\n";
  grove_oled_gosub1 += "\tHI2cOut $80, (0x01)\n";
  grove_oled_gosub1 += "\tHI2cOut $80, (0x81)\n";
  grove_oled_gosub1 += "\tHI2cOut $80, (0x53)\n";
  grove_oled_gosub1 += "\tHI2cOut $80, (0xB1)\n";
  grove_oled_gosub1 += "\tHI2cOut $80, (0x51)\n";
  grove_oled_gosub1 += "\tHI2cOut $80, (0xB3)\n";
  grove_oled_gosub1 += "\tHI2cOut $80, (0x01)\n";
  grove_oled_gosub1 += "\tHI2cOut $80, (0xB9)\n";
  grove_oled_gosub1 += "\tHI2cOut $80, (0xBC)\n";
  grove_oled_gosub1 += "\tHI2cOut $80, (0x08)\n";
  grove_oled_gosub1 += "\tHI2cOut $80, (0xBE)\n";
  grove_oled_gosub1 += "\tHI2cOut $80, (0x07)\n";
  grove_oled_gosub1 += "\tHI2cOut $80, (0xB6)\n";
  grove_oled_gosub1 += "\tHI2cOut $80, (0x01)\n";
  grove_oled_gosub1 += "\tHI2cOut $80, (0xD5)\n";
  grove_oled_gosub1 += "\tHI2cOut $80, (0x62)\n";
  grove_oled_gosub1 += "\tHI2cOut $80, (0xA4)\n";
  grove_oled_gosub1 += "\tHI2cOut $80, (0x2E)\n";
  grove_oled_gosub1 += "\tHI2cOut $80, (0xAF)\n";
  grove_oled_gosub1 += "\tpause 100\n\n";
  
  grove_oled_gosub1 += "grove_oled_clearScreen:\n";
  grove_oled_gosub1 += "\tHI2cSetup I2CMASTER, $78, I2CSLOW, I2CBYTE\n";
  grove_oled_gosub1 += "\tHI2cOut $80, (0x15)\n";
  grove_oled_gosub1 += "\tHI2cOut $80, (0x08)\n";
  grove_oled_gosub1 += "\tHI2cOut $80, (0x37)\n";
  grove_oled_gosub1 += "\tHI2cOut $80, (0x75)\n";
  grove_oled_gosub1 += "\tHI2cOut $80, (0x00)\n";
  grove_oled_gosub1 += "\tHI2cOut $80, (0x5F)\n";

  grove_oled_gosub1 += "\tFor grove_oled_row = 0 To 127 step 2\n";
  grove_oled_gosub1 += "\t\tFor grove_oled_col = 0 To 127 step 8\n";
  grove_oled_gosub1 += "\t\t\tHI2cOut $40,(0,0,0,0,0,0,0,0)\n";
  grove_oled_gosub1 += "\t\tNext\n";
  grove_oled_gosub1 += "\tNext\n";
  grove_oled_gosub1 += "\tgrove_oled_row = 0\n";
  grove_oled_gosub1 += "\tgrove_oled_col = 0\n";
  grove_oled_gosub1 += "\tReturn\n\n";

  grove_oled_gosub1 += "grove_oled_putChar:\n";
  grove_oled_gosub1 += "\tHI2cSetup I2CMASTER, $78, I2CSLOW, I2CBYTE\n";
  grove_oled_gosub1 += "\tHI2cOut $80, (0x15)\n";
  grove_oled_gosub1 += "\tgrove_w7 = grove_oled_col * 3 + 8\n";
  grove_oled_gosub1 += "\tHI2cOut $80, (grove_w7)\n";
  grove_oled_gosub1 += "\tHI2cOut $80, (0x37)\n";  
  grove_oled_gosub1 += "\tHI2cOut $80, (0x75)\n";
  grove_oled_gosub1 += "\tgrove_w7 = grove_oled_row * 8\n";
  grove_oled_gosub1 += "\tHI2cOut $80, (grove_w7)\n";
  grove_oled_gosub1 += "\tgrove_w7 = grove_w7 + 7\n";
  grove_oled_gosub1 += "\tHI2cOut $80, (grove_w7)\n";  

  grove_oled_gosub1 += "#ifndef _X2part\n";  
    grove_oled_gosub1 += "\tIf grove_oled_chr < $1A Or grove_oled_chr > $AF Then\n";
    grove_oled_gosub1 += '\t\tgrove_oled_chr = " "\n';
    grove_oled_gosub1 += "\tEnd If\n";  
  grove_oled_gosub1 += "#else\n";
    grove_oled_gosub1 += "\tIf grove_oled_chr < $1A Or grove_oled_chr > $7F Then\n";
    grove_oled_gosub1 += '\t\tgrove_oled_chr = " "\n';
    grove_oled_gosub1 += "\tEnd If\n";  
  grove_oled_gosub1 += "#endif\n";

  grove_oled_gosub1 += "\tFor grove_w5 = 0 To 5 step 2\n";
  grove_oled_gosub1 += "\t\tgrove_w6 = 1\n";
  grove_oled_gosub1 += "\t\tdo\n";
  grove_oled_gosub1 += "\t\t\tgosub grove_oled_readBit\n";
  grove_oled_gosub1 += "\t\t\tgrove_w5 = grove_w5 + 1\n";
  grove_oled_gosub1 += "\t\t\tgosub grove_oled_readBit\n";
  grove_oled_gosub1 += "\t\t\tgrove_w5 = grove_w5 - 1\n";
  grove_oled_gosub1 += "\t\t\tHI2cOut $40, (bptr)\n";
  grove_oled_gosub1 += "\t\t\tgrove_w6 = grove_w6 * 2\n";
  grove_oled_gosub1 += "\t\tloop until grove_w6 > $80\n";
  grove_oled_gosub1 += "\tNext\n";

  grove_oled_gosub1 += "\tgrove_oled_col = grove_oled_col + 1 & $0F\n";
  grove_oled_gosub1 += "\tIf grove_oled_col = 0 Then\n";
  grove_oled_gosub1 += "\t\tgrove_oled_row = grove_oled_row + 1 // 16\n";
  grove_oled_gosub1 += "\tEnd If\n";
  grove_oled_gosub1 += "\tReturn\n\n";

  grove_oled_gosub1 += "grove_oled_readBit:\n";
  grove_oled_gosub1 += "\tIf grove_w5 >= 5 then\n";
  grove_oled_gosub1 += "\t\tgrove_w7 = 0\n";
  grove_oled_gosub1 += "\telse\n";
  grove_oled_gosub1 += "\t\tIf grove_oled_chr >= $4D Then\n";
  grove_oled_gosub1 += "\t\t\tgrove_w7 = grove_oled_chr  - $4D * 5 + grove_w5\n";
  grove_oled_gosub1 += "\t\t\tReadTable grove_w7,grove_w7\n";
  grove_oled_gosub1 += "\t\tElse\n";
  grove_oled_gosub1 += "\t\t\tgrove_w7 = grove_oled_chr  - $1A * 5 + grove_w5 + 1\n";
  grove_oled_gosub1 += "\t\t\tRead grove_w7,grove_w7\n";
  grove_oled_gosub1 += "\t\tend if\n";
  grove_oled_gosub1 += "\tend if\n";
  grove_oled_gosub1 += "\tbptr = bptr * 16\n";
  grove_oled_gosub1 += "\tbptr = grove_w7 and grove_w6 max 1 * 0x0F | bptr\n";
  grove_oled_gosub1 += "\treturn\n\n";
  
  grove_oled_gosub1 += "grove_oled_putValue:\n";
  grove_oled_gosub1 += "\tif grove_oled_temp > 999 then\n";
  grove_oled_gosub1 += '\t\tgrove_oled_chr = grove_oled_temp / 10000 + "0"\n';
  grove_oled_gosub1 += "\t\tgosub grove_oled_putChar\n";
  grove_oled_gosub1 += '\t\tgrove_oled_chr = grove_oled_temp / 1000 // 10 + "0"\n';
  grove_oled_gosub1 += "\t\tgosub grove_oled_putChar\n";
  grove_oled_gosub1 += "\tend if\n";
  grove_oled_gosub1 += '\tgrove_oled_chr = grove_oled_temp / 100 // 10 + "0"\n';
  grove_oled_gosub1 += "\tgosub grove_oled_putChar\n";
  grove_oled_gosub1 += '\tgrove_oled_chr = grove_oled_temp / 10 // 10 + "0"\n';
  grove_oled_gosub1 += "\tgosub grove_oled_putChar\n";
  grove_oled_gosub1 += '\tgrove_oled_chr = grove_oled_temp // 10 + "0"\n';
  grove_oled_gosub1 += "\tgosub grove_oled_putChar\n";
  grove_oled_gosub1 += "\treturn\n\n"; 
  
  var grove_oled_gosub2 = "grove_oled_init:\n";
  grove_oled_gosub2 += "\tHI2cSetup I2CMASTER, $78, I2CSLOW, I2CBYTE\n"; 
  grove_oled_gosub2 += "\tHI2cOut $80, (0xFD)\n";
  grove_oled_gosub2 += "\tHI2cOut $80, (0x12)\n";
  grove_oled_gosub2 += "\tHI2cOut $80, (0xAE)\n";
  grove_oled_gosub2 += "\tHI2cOut $80, (0xA8)\n";
  grove_oled_gosub2 += "\tHI2cOut $80, (0x5F)\n";
  grove_oled_gosub2 += "\tHI2cOut $80, (0xA1)\n";
  grove_oled_gosub2 += "\tHI2cOut $80, (0x00)\n";
  grove_oled_gosub2 += "\tHI2cOut $80, (0xA2)\n";
  grove_oled_gosub2 += "\tHI2cOut $80, (0x60)\n";
  grove_oled_gosub2 += "\tHI2cOut $80, (0xA0)\n";  
  grove_oled_gosub2 += "\tHI2cOut $80, (0x46)\n";
  grove_oled_gosub2 += "\tHI2cOut $80, (0xAB)\n";
  grove_oled_gosub2 += "\tHI2cOut $80, (0x01)\n";
  grove_oled_gosub2 += "\tHI2cOut $80, (0x81)\n";
  grove_oled_gosub2 += "\tHI2cOut $80, (0x53)\n";
  grove_oled_gosub2 += "\tHI2cOut $80, (0xB1)\n";
  grove_oled_gosub2 += "\tHI2cOut $80, (0x51)\n";
  grove_oled_gosub2 += "\tHI2cOut $80, (0xB3)\n";
  grove_oled_gosub2 += "\tHI2cOut $80, (0x01)\n";
  grove_oled_gosub2 += "\tHI2cOut $80, (0xB9)\n";
  grove_oled_gosub2 += "\tHI2cOut $80, (0xBC)\n";
  grove_oled_gosub2 += "\tHI2cOut $80, (0x08)\n";
  grove_oled_gosub2 += "\tHI2cOut $80, (0xBE)\n";
  grove_oled_gosub2 += "\tHI2cOut $80, (0x07)\n";
  grove_oled_gosub2 += "\tHI2cOut $80, (0xB6)\n";
  grove_oled_gosub2 += "\tHI2cOut $80, (0x01)\n";
  grove_oled_gosub2 += "\tHI2cOut $80, (0xD5)\n";
  grove_oled_gosub2 += "\tHI2cOut $80, (0x62)\n";
  grove_oled_gosub2 += "\tHI2cOut $80, (0xA4)\n";
  grove_oled_gosub2 += "\tHI2cOut $80, (0x2E)\n";
  grove_oled_gosub2 += "\tHI2cOut $80, (0xAF)\n";
  grove_oled_gosub2 += "\tpause 100\n\n";
  
  grove_oled_gosub2 += "grove_oled_clearScreen:\n";
  grove_oled_gosub2 += "\tHI2cSetup I2CMASTER, $78, I2CSLOW, I2CBYTE\n";
  grove_oled_gosub2 += "\tHI2cOut $80, (0x15)\n";
  grove_oled_gosub2 += "\tHI2cOut $80, (0x08)\n";
  grove_oled_gosub2 += "\tHI2cOut $80, (0x37)\n";
  grove_oled_gosub2 += "\tHI2cOut $80, (0x75)\n";
  grove_oled_gosub2 += "\tHI2cOut $80, (0x00)\n";
  grove_oled_gosub2 += "\tHI2cOut $80, (0x5F)\n";

  grove_oled_gosub2 += "\tFor grove_oled_row = 0 To 95 step 2\n";
  grove_oled_gosub2 += "\t\tFor grove_oled_col = 0 To 95 step 8\n";
  grove_oled_gosub2 += "\t\t\tHI2cOut $40,(0,0,0,0,0,0,0,0)\n";
  grove_oled_gosub2 += "\t\tNext\n";
  grove_oled_gosub2 += "\tNext\n";
  grove_oled_gosub2 += "\tgrove_oled_row = 0\n";
  grove_oled_gosub2 += "\tgrove_oled_col = 0\n";
  grove_oled_gosub2 += "\tReturn\n\n";

  grove_oled_gosub2 += "grove_oled_putChar:\n";
  grove_oled_gosub2 += "\tHI2cSetup I2CMASTER, $78, I2CSLOW, I2CBYTE\n";
  grove_oled_gosub2 += "\tHI2cOut $80, (0x15)\n";
  grove_oled_gosub2 += "\tgrove_w7 = grove_oled_col * 3 + 8\n";
  grove_oled_gosub2 += "\tHI2cOut $80, (grove_w7)\n";
  grove_oled_gosub2 += "\tHI2cOut $80, (0x37)\n";  
  grove_oled_gosub2 += "\tHI2cOut $80, (0x75)\n";
  grove_oled_gosub2 += "\tgrove_w7 = grove_oled_row * 8\n";
  grove_oled_gosub2 += "\tHI2cOut $80, (grove_w7)\n";
  grove_oled_gosub2 += "\tgrove_w7 = grove_w7 + 7\n";
  grove_oled_gosub2 += "\tHI2cOut $80, (grove_w7)\n";  

  grove_oled_gosub2 += "#ifndef _X2part\n";  
    grove_oled_gosub2 += "\tIf grove_oled_chr < $1A Or grove_oled_chr > $AF Then\n";
    grove_oled_gosub2 += '\t\tgrove_oled_chr = " "\n';
    grove_oled_gosub2 += "\tEnd If\n";  
  grove_oled_gosub2 += "#else\n";
    grove_oled_gosub2 += "\tIf grove_oled_chr < $1A Or grove_oled_chr > $7F Then\n";
    grove_oled_gosub2 += '\t\tgrove_oled_chr = " "\n';
    grove_oled_gosub2 += "\tEnd If\n";  
  grove_oled_gosub2 += "#endif\n";

  grove_oled_gosub2 += "\tFor grove_w5 = 0 To 5 step 2\n";
  grove_oled_gosub2 += "\t\tgrove_w6 = 1\n";
  grove_oled_gosub2 += "\t\tdo\n";
  grove_oled_gosub2 += "\t\t\tgosub grove_oled_readBit\n";
  grove_oled_gosub2 += "\t\t\tgrove_w5 = grove_w5 + 1\n";
  grove_oled_gosub2 += "\t\t\tgosub grove_oled_readBit\n";
  grove_oled_gosub2 += "\t\t\tgrove_w5 = grove_w5 - 1\n";
  grove_oled_gosub2 += "\t\t\tHI2cOut $40, (bptr)\n";
  grove_oled_gosub2 += "\t\t\tgrove_w6 = grove_w6 * 2\n";
  grove_oled_gosub2 += "\t\tloop until grove_w6 > $80\n";
  grove_oled_gosub2 += "\tNext\n";

  grove_oled_gosub2 += "\tgrove_oled_col = grove_oled_col + 1 & $0F\n";
  grove_oled_gosub2 += "\tIf grove_oled_col = 0 Then\n";
  grove_oled_gosub2 += "\t\tgrove_oled_row = grove_oled_row + 1 // 12\n";
  grove_oled_gosub2 += "\tEnd If\n";
  grove_oled_gosub2 += "\tReturn\n\n";

  grove_oled_gosub2 += "grove_oled_readBit:\n";
  grove_oled_gosub2 += "\tIf grove_w5 >= 5 then\n";
  grove_oled_gosub2 += "\t\tgrove_w7 = 0\n";
  grove_oled_gosub2 += "\telse\n";
  grove_oled_gosub2 += "\t\tIf grove_oled_chr >= $4D Then\n";
  grove_oled_gosub2 += "\t\t\tgrove_w7 = grove_oled_chr  - $4D * 5 + grove_w5\n";
  grove_oled_gosub2 += "\t\t\tReadTable grove_w7,grove_w7\n";
  grove_oled_gosub2 += "\t\tElse\n";
  grove_oled_gosub2 += "\t\t\tgrove_w7 = grove_oled_chr  - $1A * 5 + grove_w5 + 1\n";
  grove_oled_gosub2 += "\t\t\tRead grove_w7,grove_w7\n";
  grove_oled_gosub2 += "\t\tend if\n";
  grove_oled_gosub2 += "\tend if\n";
  grove_oled_gosub2 += "\tbptr = bptr * 16\n";
  grove_oled_gosub2 += "\tbptr = grove_w7 and grove_w6 max 1 * 0x0F | bptr\n";
  grove_oled_gosub2 += "\treturn\n\n";
  
  grove_oled_gosub2 += "grove_oled_putValue:\n";
  grove_oled_gosub2 += "\tif grove_oled_temp > 999 then\n";
  grove_oled_gosub2 += '\t\tgrove_oled_chr = grove_oled_temp / 10000 + "0"\n';
  grove_oled_gosub2 += "\t\tgosub grove_oled_putChar\n";
  grove_oled_gosub2 += '\t\tgrove_oled_chr = grove_oled_temp / 1000 // 10 + "0"\n';
  grove_oled_gosub2 += "\t\tgosub grove_oled_putChar\n";
  grove_oled_gosub2 += "\tend if\n";
  grove_oled_gosub2 += '\tgrove_oled_chr = grove_oled_temp / 100 // 10 + "0"\n';
  grove_oled_gosub2 += "\tgosub grove_oled_putChar\n";
  grove_oled_gosub2 += '\tgrove_oled_chr = grove_oled_temp / 10 // 10 + "0"\n';
  grove_oled_gosub2 += "\tgosub grove_oled_putChar\n";
  grove_oled_gosub2 += '\tgrove_oled_chr = grove_oled_temp // 10 + "0"\n';
  grove_oled_gosub2 += "\tgosub grove_oled_putChar\n";
  grove_oled_gosub2 += "\treturn\n\n";
   
  var grove_oled_table = "";  
  grove_oled_table += "eeprom $00, ( $00                 )   ; Unused\n";
  grove_oled_table += "eeprom $01, ( $7F,$3E,$1C,$08,$00 )   ; $1A - Right Arrow\n";
  grove_oled_table += "eeprom $06, ( $00,$08,$1C,$3E,$7F )   ; $1B - Left Arrow\n";
  grove_oled_table += "eeprom $0B, ( $08,$0C,$0E,$0C,$08 )   ; $1C - Up Arrow\n";
  grove_oled_table += "eeprom $10, ( $08,$18,$38,$18,$08 )   ; $1D - Down Arrow\n";
  grove_oled_table += "eeprom $15, ( $48,$7E,$49,$4A,$00 )   ; $1E - Pound\n";
  grove_oled_table += "eeprom $1A, ( $00,$02,$05,$02,$00 )   ; $1F - Degree\n";
  grove_oled_table += "eeprom $1F, ( $00,$00,$00,$00,$00 )   ; $20 - Space\n";
  grove_oled_table += "eeprom $24, ( $00,$00,$2F,$00,$00 )   ; $21 - Exclamation mark\n";
  grove_oled_table += "eeprom $29, ( $00,$07,$00,$07,$00 )   ; $22 - Double quote\n";
  grove_oled_table += "eeprom $2E, ( $14,$7F,$14,$7F,$14 )   ; $23 - Hash\n";
  grove_oled_table += "eeprom $33, ( $24,$2A,$7F,$2A,$12 )   ; $24 - Dollar sign\n";
  grove_oled_table += "eeprom $38, ( $23,$13,$08,$64,$62 )   ; $25 - Percent\n";
  grove_oled_table += "eeprom $3D, ( $36,$49,$55,$22,$50 )   ; $26 - Ampersand\n";
  grove_oled_table += "eeprom $42, ( $00,$05,$03,$00,$00 )   ; $27 - Single quote\n";
  grove_oled_table += "eeprom $47, ( $00,$1C,$22,$41,$00 )   ; $28 - (\n";
  grove_oled_table += "eeprom $4C, ( $00,$41,$22,$1C,$00 )   ; $29 - )\n";
  grove_oled_table += "eeprom $51, ( $14,$08,$3E,$08,$14 )   ; $2A - Star / Astrix\n";
  grove_oled_table += "eeprom $56, ( $08,$08,$3E,$08,$08 )   ; $2B - Plus\n";
  grove_oled_table += "eeprom $5B, ( $00,$00,$58,$38,$00 )   ; $2C - Comma\n";
  grove_oled_table += "eeprom $60, ( $10,$10,$10,$10,$10 )   ; $2D - Minus\n";
  grove_oled_table += "eeprom $65, ( $00,$60,$60,$00,$00 )   ; $2E - Period / Full Stop\n";
  grove_oled_table += "eeprom $6A, ( $20,$10,$08,$04,$02 )   ; $2F - /\n";
  grove_oled_table += "eeprom $6F, ( $3E,$51,$49,$45,$3E )   ; $30 - 0\n";
  grove_oled_table += "eeprom $74, ( $00,$42,$7F,$40,$00 )   ; $31 - 1\n";
  grove_oled_table += "eeprom $79, ( $42,$61,$51,$49,$46 )   ; $32 - 2\n";
  grove_oled_table += "eeprom $7E, ( $21,$41,$45,$4B,$31 )   ; $33 - 3\n";
  grove_oled_table += "eeprom $83, ( $18,$14,$12,$7F,$10 )   ; $34 - 4\n";
  grove_oled_table += "eeprom $88, ( $27,$45,$45,$45,$39 )   ; $35 - 5\n";
  grove_oled_table += "eeprom $8D, ( $3C,$4A,$49,$49,$30 )   ; $36 - 6\n";
  grove_oled_table += "eeprom $92, ( $01,$71,$09,$05,$03 )   ; $37 - 7\n";
  grove_oled_table += "eeprom $97, ( $36,$49,$49,$49,$36 )   ; $38 - 8\n";
  grove_oled_table += "eeprom $9C, ( $06,$49,$49,$29,$1E )   ; $39 - 9\n";
  grove_oled_table += "eeprom $A1, ( $00,$36,$36,$00,$00 )   ; $3A - Colon\n";
  grove_oled_table += "eeprom $A6, ( $00,$56,$36,$00,$00 )   ; $3B - Semicolon\n";
  grove_oled_table += "eeprom $AB, ( $08,$14,$22,$41,$00 )   ; $3C - <\n";
  grove_oled_table += "eeprom $B0, ( $14,$14,$14,$14,$14 )   ; $3D - =\n";
  grove_oled_table += "eeprom $B5, ( $00,$41,$22,$14,$08 )   ; $3E - >\n";
  grove_oled_table += "eeprom $BA, ( $02,$01,$51,$09,$06 )   ; $3F - ?\n";
  grove_oled_table += "eeprom $BF, ( $3E,$41,$19,$25,$3E )   ; $40 - @\n";
  grove_oled_table += "eeprom $C4, ( $7E,$11,$11,$11,$7E )   ; $41 - A\n";
  grove_oled_table += "eeprom $C9, ( $7F,$49,$49,$49,$36 )   ; $42 - B\n";
  grove_oled_table += "eeprom $CE, ( $3E,$41,$41,$41,$22 )   ; $43 - C\n";
  grove_oled_table += "eeprom $D3, ( $7F,$41,$41,$22,$1C )   ; $44 - D\n";
  grove_oled_table += "eeprom $D8, ( $7F,$49,$49,$49,$41 )   ; $45 - E\n";
  grove_oled_table += "eeprom $DD, ( $7F,$09,$09,$09,$01 )   ; $46 - F\n";
  grove_oled_table += "eeprom $E2, ( $3E,$41,$49,$49,$7A )   ; $47 - G\n";
  grove_oled_table += "eeprom $E7, ( $7F,$08,$08,$08,$7F )   ; $48 - H\n";
  grove_oled_table += "eeprom $EC, ( $00,$41,$7F,$41,$00 )   ; $49 - I\n";
  grove_oled_table += "eeprom $F1, ( $20,$40,$41,$3F,$01 )   ; $4A - J\n";
  grove_oled_table += "eeprom $F6, ( $7F,$08,$14,$22,$41 )   ; $4B - K\n";
  grove_oled_table += "eeprom $FB, ( $7F,$40,$40,$40,$40 )   ; $4C - L\n";

  grove_oled_table += "table $00, ( $7F,$02,$0C,$02,$7F )   ; $4D - M\n";
  grove_oled_table += "table $05, ( $7F,$04,$08,$10,$7F )   ; $4E - N\n";
  grove_oled_table += "table $0A, ( $3E,$41,$41,$41,$3E )   ; $4F - O\n";
  grove_oled_table += "table $0F, ( $7F,$09,$09,$09,$06 )   ; $50 - P\n";
  grove_oled_table += "table $14, ( $3E,$41,$51,$21,$5E )   ; $51 - Q\n";
  grove_oled_table += "table $19, ( $7F,$09,$19,$29,$46 )   ; $52 - R\n";
  grove_oled_table += "table $1E, ( $46,$49,$49,$49,$31 )   ; $53 - S\n";
  grove_oled_table += "table $23, ( $01,$01,$7F,$01,$01 )   ; $54 - T\n";
  grove_oled_table += "table $28, ( $3F,$40,$40,$40,$3F )   ; $55 - U\n";
  grove_oled_table += "table $2D, ( $1F,$20,$40,$20,$1F )   ; $56 - V\n";
  grove_oled_table += "table $32, ( $3F,$40,$38,$40,$3F )   ; $57 - W\n";
  grove_oled_table += "table $37, ( $63,$14,$08,$14,$63 )   ; $58 - X\n";
  grove_oled_table += "table $3C, ( $07,$08,$70,$08,$07 )   ; $59 - Y\n";
  grove_oled_table += "table $41, ( $61,$51,$49,$45,$43 )   ; $5A - Z\n";
  grove_oled_table += "table $46, ( $00,$7F,$41,$41,$00 )   ; $5B - [\n";
  grove_oled_table += "table $4B, ( $02,$04,$08,$10,$20 )   ; $5C - Backslash\n";
  grove_oled_table += "table $50, ( $00,$41,$41,$7F,$00 )   ; $5D - ]\n";
  grove_oled_table += "table $55, ( $04,$02,$01,$02,$04 )   ; $5E - Up arrow\n";
  grove_oled_table += "table $5A, ( $40,$40,$40,$40,$40 )   ; $5F - Underscore\n";
  grove_oled_table += "table $5F, ( $00,$01,$02,$04,$00 )   ; $60 - Back tick\n";
  grove_oled_table += "table $64, ( $20,$54,$54,$54,$78 )   ; $61 - a\n";
  grove_oled_table += "table $69, ( $7F,$48,$44,$44,$38 )   ; $62 - b\n";
  grove_oled_table += "table $6E, ( $38,$44,$44,$44,$20 )   ; $63 - c\n";
  grove_oled_table += "table $73, ( $38,$44,$44,$48,$7F )   ; $64 - d\n";
  grove_oled_table += "table $78, ( $38,$54,$54,$54,$18 )   ; $65 - e\n";
  grove_oled_table += "table $7D, ( $08,$7E,$09,$01,$02 )   ; $66 - f\n";
  grove_oled_table += "table $82, ( $0C,$52,$52,$52,$3E )   ; $67 - g\n";
  grove_oled_table += "table $87, ( $7F,$08,$04,$04,$78 )   ; $68 - h\n";
  grove_oled_table += "table $8C, ( $00,$44,$7D,$40,$00 )   ; $69 - i\n";
  grove_oled_table += "table $91, ( $20,$40,$44,$3D,$00 )   ; $6A - j\n";
  grove_oled_table += "table $96, ( $7F,$10,$28,$44,$00 )   ; $6B - k\n";
  grove_oled_table += "table $9B, ( $00,$41,$7F,$40,$00 )   ; $6C - l\n";
  grove_oled_table += "table $A0, ( $7C,$04,$18,$04,$78 )   ; $6D - m\n";
  grove_oled_table += "table $A5, ( $7C,$08,$04,$04,$78 )   ; $6E - n\n";
  grove_oled_table += "table $AA, ( $38,$44,$44,$44,$38 )   ; $6F - o\n";
  grove_oled_table += "table $AF, ( $7C,$14,$14,$14,$08 )   ; $70 - p\n";
  grove_oled_table += "table $B4, ( $08,$14,$14,$18,$7C )   ; $71 - q\n";
  grove_oled_table += "table $B9, ( $7C,$08,$04,$04,$08 )   ; $72 - r\n";
  grove_oled_table += "table $BE, ( $48,$54,$54,$54,$20 )   ; $73 - s\n";
  grove_oled_table += "table $C3, ( $04,$3F,$44,$40,$20 )   ; $74 - t\n";
  grove_oled_table += "table $C8, ( $3C,$40,$40,$20,$7C )   ; $75 - u\n";
  grove_oled_table += "table $CD, ( $1C,$20,$40,$20,$1C )   ; $76 - v\n";
  grove_oled_table += "table $D2, ( $3C,$40,$30,$40,$3C )   ; $77 - w\n";
  grove_oled_table += "table $D7, ( $44,$28,$10,$28,$44 )   ; $78 - x\n";
  grove_oled_table += "table $DC, ( $0C,$50,$50,$50,$3C )   ; $79 - y\n";
  grove_oled_table += "table $E1, ( $44,$64,$54,$4C,$44 )   ; $7A - z\n";
  grove_oled_table += "table $E6, ( $00,$08,$36,$41,$00 )   ; $7B - {\n";
  grove_oled_table += "table $EB, ( $00,$00,$7F,$00,$00 )   ; $7C - Vertical bar\n";
  grove_oled_table += "table $F0, ( $00,$41,$36,$08,$00 )   ; $7D - }\n";
  grove_oled_table += "table $F5, ( $04,$02,$04,$08,$04 )   ; $7E - Tilde\n";
  grove_oled_table += "table $FA, ( $7F,$41,$41,$41,$7F )   ; $7F - Rectangle\n";

  grove_oled_table += "#ifndef _X2part\n";  
    grove_oled_table += "table  $FF, ( $0E,$51,$51,$71,$11 )  ; $80 - C,\n";
    grove_oled_table += "table $104, ( $38,$42,$40,$42,$38 )  ; $81 - u:\n";
    grove_oled_table += "table $109, ( $38,$54,$56,$55,$18 )  ; $82 - e/\n";
    grove_oled_table += "table $10E, ( $30,$56,$55,$56,$78 )  ; $83 - a^\n";
    grove_oled_table += "table $113, ( $30,$55,$54,$55,$78 )  ; $84 - a:\n";
    grove_oled_table += "table $118, ( $30,$55,$56,$54,$78 )  ; $85 - a`\n";
    grove_oled_table += "table $11D, ( $30,$54,$55,$54,$78 )  ; $86 - a.\n";
    grove_oled_table += "table $122, ( $0E,$51,$71,$11,$00 )  ; $87 - c,\n";
    grove_oled_table += "table $127, ( $38,$56,$55,$56,$18 )  ; $88 - e^\n";
    grove_oled_table += "table $12C, ( $38,$55,$54,$55,$18 )  ; $89 - e:\n";
    grove_oled_table += "table $131, ( $38,$55,$56,$54,$18 )  ; $8A - e`\n";
    grove_oled_table += "table $136, ( $00,$49,$78,$41,$00 )  ; $8B - i:\n";
    grove_oled_table += "table $13B, ( $00,$4A,$79,$42,$00 )  ; $8C - i^\n";
    grove_oled_table += "table $140, ( $00,$49,$7A,$40,$00 )  ; $8D - i`\n";
    grove_oled_table += "table $145, ( $78,$15,$14,$15,$78 )  ; $8E - A:\n";
    grove_oled_table += "table $14A, ( $70,$2A,$2D,$2A,$70 )  ; $8F - Ao\n";
    grove_oled_table += "table $14F, ( $7C,$54,$56,$55,$44 )  ; $90 - E/\n";
    grove_oled_table += "table $154, ( $09,$15,$1E,$15,$16 )  ; $91 - s reversed\n";
    grove_oled_table += "table $159, ( $7E,$09,$0F,$09,$01 )  ; $92 - fF\n";
    grove_oled_table += "table $15E, ( $38,$46,$45,$46,$38 )  ; $93 - O^\n";
    grove_oled_table += "table $163, ( $30,$49,$48,$49,$30 )  ; $94 - o:\n";
    grove_oled_table += "table $168, ( $30,$49,$4A,$48,$30 )  ; $95 - o`\n";
    grove_oled_table += "table $16D, ( $38,$42,$41,$22,$78 )  ; $96 - u^\n";
    grove_oled_table += "table $172, ( $38,$41,$42,$20,$78 )  ; $97 - u`\n";
    grove_oled_table += "table $177, ( $3C,$41,$42,$20,$78 )  ; $98 - U`\n";
    grove_oled_table += "table $17C, ( $0C,$51,$50,$51,$3C )  ; $99 - y:\n";
    grove_oled_table += "table $181, ( $3D,$42,$42,$42,$3D )  ; $9A - O:\n";
    grove_oled_table += "table $186, ( $79,$0A,$12,$22,$79 )  ; $9B - N~\n";
    grove_oled_table += "table $18B, ( $79,$12,$0A,$0A,$71 )  ; $9C - n~\n";
    grove_oled_table += "table $190, ( $48,$55,$55,$55,$5E )  ; $9D - a_\n";
    grove_oled_table += "table $195, ( $4E,$51,$51,$51,$4E )  ; $9E - o_\n";
    grove_oled_table += "table $19A, ( $30,$48,$45,$40,$20 )  ; $9F - ? inverted\n";
    grove_oled_table += "table $19F, ( $20,$54,$56,$55,$78 )  ; $A0 - a/\n";
    grove_oled_table += "table $1A4, ( $00,$48,$7A,$41,$00 )  ; $A1 - i/\n";
    grove_oled_table += "table $1A9, ( $30,$48,$4A,$49,$30 )  ; $A2 - o/\n";
    grove_oled_table += "table $1AE, ( $38,$40,$42,$21,$78 )  ; $A3 - u/\n";
    grove_oled_table += "table $1B3, ( $0C,$12,$3F,$12,$12 )  ; $A4 - c| cent\n";
    grove_oled_table += "table $1B8, ( $48,$7E,$49,$41,$42 )  ; $A5 - ? pound\n";
    grove_oled_table += "table $1BD, ( $15,$16,$7C,$16,$15 )  ; $A6 - Y yen\n";
    grove_oled_table += "table $1C2, ( $7F,$05,$15,$7A,$50 )  ; $A7 - Pt\n";
    grove_oled_table += "table $1C7, ( $40,$48,$3E,$09,$01 )  ; $A8 - f\n";
    grove_oled_table += "table $1CC, ( $00,$00,$7A,$00,$00 )  ; $A9 - i\n";
    grove_oled_table += "table $1D1, ( $72,$29,$29,$2A,$71 )  ; $AA - A~\n";
    grove_oled_table += "table $1D6, ( $22,$51,$55,$56,$79 )  ; $AB - a~\n";
    grove_oled_table += "table $1DB, ( $32,$49,$49,$4A,$31 )  ; $AC - o~\n";
    grove_oled_table += "table $1E0, ( $3A,$45,$45,$46,$39 )  ; $AD - O~\n";
    grove_oled_table += "table $1E5, ( $5C,$32,$2A,$26,$1D )  ; $AE - O slashed\n";
    grove_oled_table += "table $1EA, ( $58,$24,$3C,$24,$1A )  ; $AF - o slashed\n";
  grove_oled_table += "#endif\n";  

generators['grove_oled_print'] = function (block) {
  if (!generators.definitions_['grove_define_x2part']) generators.definitions_['grove_define_x2part'] = x2part_header;    
  if (!generators.definitions_['grove_symbol_header']) generators.definitions_['grove_symbol_header'] = grove_symbol_header;
  var type = block.getFieldValue('TYPE');
  if (!generators.definitions_['grove_oled']) generators.definitions_['grove_oled'] = grove_oled_header;
  if (type == 1) { //128x128
    if (!generators.gosubs_['grove_oled']) generators.gosubs_['grove_oled'] = grove_oled_gosub1;         
  } else if (type == 2) { //96x96
    if (!generators.gosubs_['grove_oled']) generators.gosubs_['grove_oled'] = grove_oled_gosub2;   
  } else { //128x64
    if (!generators.gosubs_['grove_oled']) generators.gosubs_['grove_oled'] = grove_oled_gosub;      
  }
  if (!generators.gosubs_['grove_oled_table']) generators.gosubs_['grove_oled_table'] = grove_oled_table;    

  var value = generators.valueToCode(block, 'VALUE', generators.ORDER_ATOMIC);
  var ascii = (block.getFieldValue('ASCII') == 'TRUE');

  var code = "";
  if ( value[0] != '"' && ascii) {
    code += "\tgrove_oled_temp = " + value + "\n";
    code += "\tgosub grove_oled_putValue\n";
  } else if ( value[0] != '"' ) {
    code += "\tgrove_oled_chr = " + value + "\n";
    code += "\tgosub grove_oled_putChar\n";
  } else if (value.length == 3) {
    // Optimise single character string, no loop required for, eg "x"
    value = generators.convertAnsiiToGroveChars(value);
    code += "\tgrove_oled_chr = " + value + "\n";
    code += "\tgosub grove_oled_putChar\n";
  } else {
    value = generators.convertAnsiiToGroveChars(value);
    code += "\tgrove_oled_temp = 0\n";
    code += "\tdo\n";
    code += "\t\tlookup grove_oled_temp,(" + value + ",0),grove_oled_chr\n";
    code += "\t\tif grove_oled_chr <> 0 then\n";
    code += "\t\t\tgosub grove_oled_putChar\n";
    code += "\t\t\tinc grove_oled_temp\n";
    code += "\t\tend if\n";
    code += "\tloop until grove_oled_chr = 0\n";
  }
  return generators.wrapOutput(block, code);
};

generators['grove_oled_plot'] = function (block) {
  if (!generators.definitions_['grove_define_x2part']) generators.definitions_['grove_define_x2part'] = x2part_header;    
  if (!generators.definitions_['grove_symbol_header']) generators.definitions_['grove_symbol_header'] = grove_symbol_header;
  var type = block.getFieldValue('TYPE');
  if (!generators.definitions_['grove_oled']) generators.definitions_['grove_oled'] = grove_oled_header;
  if (type == 1) { //128x128
    if (!generators.gosubs_['grove_oled']) generators.gosubs_['grove_oled'] = grove_oled_gosub1;         
  } else if (type == 2) { //96x96
    if (!generators.gosubs_['grove_oled']) generators.gosubs_['grove_oled'] = grove_oled_gosub2;   
  } else { //128x64
    if (!generators.gosubs_['grove_oled']) generators.gosubs_['grove_oled'] = grove_oled_gosub;      
  }
  if (!generators.gosubs_['grove_oled_table']) generators.gosubs_['grove_oled_table'] = grove_oled_table;   
  
  var row = generators.valueToCode(block, 'ROW', generators.ORDER_ATOMIC);
  var col = generators.valueToCode(block, 'COL', generators.ORDER_ATOMIC);
  var type = block.getFieldValue('TYPE');
  var code = "\tgrove_oled_col = " + col + " max 16 - 1\n";
  code += "\tgrove_oled_row = " + row + " max 16 - 1\n";
  return generators.wrapOutput(block, code);
};

generators['grove_oled_command'] = function (block) {
  if (!generators.definitions_['grove_define_x2part']) generators.definitions_['grove_define_x2part'] = x2part_header;    
  if (!generators.definitions_['grove_symbol_header']) generators.definitions_['grove_symbol_header'] = grove_symbol_header;
  var type = block.getFieldValue('TYPE');
  if (!generators.definitions_['grove_oled']) generators.definitions_['grove_oled'] = grove_oled_header;
  if (type == 1) { //128x128
    if (!generators.gosubs_['grove_oled']) generators.gosubs_['grove_oled'] = grove_oled_gosub1;         
  } else if (type == 2) { //96x96
    if (!generators.gosubs_['grove_oled']) generators.gosubs_['grove_oled'] = grove_oled_gosub2;   
  } else { //128x64
    if (!generators.gosubs_['grove_oled']) generators.gosubs_['grove_oled'] = grove_oled_gosub;      
  }
  if (!generators.gosubs_['grove_oled_table']) generators.gosubs_['grove_oled_table'] = grove_oled_table;   
  
  var codetag = block.getFieldValue('CODETAG');
  var code = "";
  if (codetag == "0xFF") {
    code += "\tgosub grove_oled_clearScreen\n";
  } else {
    code += "\tgrove_oled_row = " + codetag + "\n";
    code += "\tgrove_oled_col = 0\n";
  }
  return generators.wrapOutput(block, code);
};

var rgb_lcd_header =  '\t;Initialise RGB LCD module\n';
rgb_lcd_header += '\thi2csetup I2CMASTER, $7C, I2CSLOW, I2CBYTE\n';
rgb_lcd_header += '\tpause 50\n';
rgb_lcd_header += '\thi2cOut $80,($38)\n';
rgb_lcd_header += '\tpause 5\n';
rgb_lcd_header += '\thi2cout $80,($38)\n';
rgb_lcd_header += '\thi2cout $80,($38)\n';
rgb_lcd_header += '\thi2cout $80,($0C)\n';
rgb_lcd_header += '\thi2cout $80,($06)\n';
rgb_lcd_header += '\thi2cout $80,($01)\n';
rgb_lcd_header += '\tpause 10\n';
rgb_lcd_header += '\thi2cout $80,($02)\n';
rgb_lcd_header += '\tpause 10\n';
rgb_lcd_header += '\thi2csetup I2CMASTER, $C4, I2CSLOW, I2CBYTE\n';
rgb_lcd_header += '\thi2cOut $00,($00)\n';
rgb_lcd_header += '\thi2cout $01,($20)\n';
rgb_lcd_header += '\thi2cout $02,($FF)\n';
rgb_lcd_header += '\thi2cout $03,($FF)\n';
rgb_lcd_header += '\thi2cout $04,($FF)\n';
rgb_lcd_header += '\thi2cout $06,($FF)\n';
rgb_lcd_header += '\thi2cout $07,($00)\n';
rgb_lcd_header += '\thi2cout $08,($FF)\n';

generators['grove_rgb_lcd_print'] = function (block) {
  if (!generators.definitions_['grove_symbol_header']) generators.definitions_['grove_symbol_header'] = grove_symbol_header;
  if (!generators.definitions_['grove_rgb_lcd_init']) generators.definitions_['grove_rgb_lcd_init'] = rgb_lcd_header;
  var value = generators.valueToCode(block, 'VALUE', generators.ORDER_ATOMIC);
  var ascii = (block.getFieldValue('ASCII') == 'TRUE');
  
  var code = "\thi2csetup I2CMASTER, $7C, I2CSLOW, I2CBYTE\n";
  if ( value[0] != '"' && ascii) {
    code += "\tgrove_w1 = " + value + "\n";
    code += "\tif grove_w1 > 999 then\n";
    code += "\t\tbintoascii grove_w1, grove_w2, grove_w3, grove_w4, grove_w5, grove_w6 \n";
    code += "\t\thi2cout $40, (grove_w2, grove_w3, grove_w4, grove_w5, grove_w6 )\n";
    code += "\telse\n";
    code += "\t\tbintoascii grove_w1, grove_w2, grove_w3, grove_w4\n";
    code += "\t\thi2cout $40, (grove_w2, grove_w3, grove_w4 )\n";
    code += "\tend if\n";
  } else if ( value[0] != '"' ) {
    code += "\thi2cout $40, ( " + value + " )\n";
  } else {
    value = generators.convertAnsiiToLcdChars(value);
    code += "\thi2cout $40, ( " + value + " )\n";
  }
  return generators.wrapOutput(block, code);
};

generators['grove_rgb_lcd_command'] = function (block) {
  if (!generators.definitions_['grove_symbol_header']) generators.definitions_['grove_symbol_header'] = grove_symbol_header;
  if (!generators.definitions_['grove_rgb_lcd_init']) generators.definitions_['grove_rgb_lcd_init'] = rgb_lcd_header;
  var codetag = block.getFieldValue('CODETAG');
  var code = "\thi2csetup I2CMASTER, $7C, I2CSLOW, I2CBYTE\n";
  //codetag can be a single byte or a complete phrase
  if (codetag.length > 4) {
    code += "\t" + codetag + "\n";
  } else {
    code += "\thi2cout $80, (" + codetag + " )\n";
    if (codetag == "0x01") {
      code += "\tpause 10\n";
    }
  }
  return generators.wrapOutput(block, code);
};

generators['grove_rgb_lcd_color'] = function (block) {
  if (!generators.definitions_['grove_symbol_header']) generators.definitions_['grove_symbol_header'] = grove_symbol_header;
  if (!generators.definitions_['grove_rgb_lcd_init']) generators.definitions_['grove_rgb_lcd_init'] = rgb_lcd_header;
  var colour_rgb = block.getFieldValue('RGB');
  var code = "\thi2csetup I2CMASTER, $C4, I2CSLOW, I2CBYTE\n";
  code +=    "\thi2cout $04, ( " + hexToR(colour_rgb) + " )\n";
  code +=    "\thi2cout $03, ( " + hexToG(colour_rgb) + " )\n";
  code +=    "\thi2cout $02, ( " + hexToB(colour_rgb) + " )\n";
  return generators.wrapOutput(block, code);
};

var grove_4_digit_gosub =  'grove_4digit_updateDisplay:\n';
  grove_4_digit_gosub += '\tif grove_4digit_brightness  <= 7 then\n';
  grove_4_digit_gosub += '\t\tgrove_w5 = $40                       : gosub grove_4digit_sendCommand\n';
  grove_4_digit_gosub += '\t\tgrove_w5 = $C0                       : gosub grove_4digit_sendCommand\n';
  grove_4_digit_gosub += '\t\tgrove_w5 = grove_4digit_value / 1000 : gosub grove_4digit_sendDigit\n';
  grove_4_digit_gosub += '\t\tgrove_w5 = grove_4digit_value /  100 : gosub grove_4digit_sendDigit\n';
  grove_4_digit_gosub += '\t\tgrove_w5 = grove_4digit_value /   10 : gosub grove_4digit_sendDigit\n';
  grove_4_digit_gosub += '\t\tgrove_w5 = grove_4digit_value        : gosub grove_4digit_sendDigit\n';
  grove_4_digit_gosub += '\tend if\n';
  grove_4_digit_gosub += '\tgrove_w5 = grove_4digit_brightness ^ $88\n';
  grove_4_digit_gosub += '\tgosub grove_4digit_sendCommand\n';
  grove_4_digit_gosub += '\thigh grove_4digit_scl\n';
  grove_4_digit_gosub += '\thigh grove_4digit_sda\n';
  grove_4_digit_gosub += '\treturn\n\n';

  grove_4_digit_gosub += 'grove_4digit_sendCommand:\n';
  grove_4_digit_gosub += '\thigh grove_4digit_scl\n';
  grove_4_digit_gosub += '\thigh grove_4digit_sda\n';
  grove_4_digit_gosub += '\tlow  grove_4digit_sda\n';
  grove_4_digit_gosub += '\tlow  grove_4digit_scl\n';
  grove_4_digit_gosub += '\tgoto grove_4digit_sendByte\n\n';

  grove_4_digit_gosub += 'grove_4digit_sendDigit:\n';
  grove_4_digit_gosub += '\tgrove_w5 = grove_w5 // 10\n';
  grove_4_digit_gosub += '\tlookup grove_w5,($3F,$06,$5B,$4F,$66,$6D,$7D,$07,$7F,$6F),grove_w5\n';
  grove_4_digit_gosub += '\tif grove_4digit_value >= 10000 then\n';
  grove_4_digit_gosub += '\t\tgrove_w5 = grove_w5 Or $80\n';
  grove_4_digit_gosub += '\tend if\n\n';

  grove_4_digit_gosub += 'grove_4digit_sendByte:\n';
  grove_4_digit_gosub += '\tgrove_w5 = grove_w5 Or $8000\n';
  grove_4_digit_gosub += '\tdo\n';
  grove_4_digit_gosub += '\t\tgrove_w6 = grove_w5 And 1 \n';
  grove_4_digit_gosub += '\t\tif grove_w6 = 1 Then\n';
  grove_4_digit_gosub += '\t\t\thigh grove_4digit_sda\n';
  grove_4_digit_gosub += '\t\telse\n';
  grove_4_digit_gosub += '\t\t\tlow  grove_4digit_sda\n';
  grove_4_digit_gosub += '\t\tend if\n';
  grove_4_digit_gosub += '\t\thigh grove_4digit_scl\n';
  grove_4_digit_gosub += '\t\tlow grove_4digit_scl\n';
  grove_4_digit_gosub += '\t\tgrove_w5 = grove_w5 / 2\n';
  grove_4_digit_gosub += '\tloop until grove_w5 <= $0080\n';
  grove_4_digit_gosub += '\tinput grove_4digit_sda\n';
  grove_4_digit_gosub += '\thigh grove_4digit_scl\n';
  grove_4_digit_gosub += '\tlow grove_4digit_scl\n';
  grove_4_digit_gosub += '\tlow grove_4digit_sda\n';
  grove_4_digit_gosub += '\treturn\n';

  var grove_4_digit_header = 'symbol grove_4digit_brightness = grove_w1\n';
  grove_4_digit_header += 'symbol grove_4digit_value = grove_w2\n';
  grove_4_digit_header += 'symbol grove_4digit_scl = grove_w3\n';
  grove_4_digit_header += 'symbol grove_4digit_sda = grove_w4\n';

generators['grove_4_digit_display'] = function (block) {
  if (!generators.definitions_['grove_symbol_header']) generators.definitions_['grove_symbol_header'] = grove_symbol_header;
  if (!generators.definitions_['grove_4_digit']) generators.definitions_['grove_4_digit'] = grove_4_digit_header;
  if (!generators.gosubs_['grove_4_digit']) generators.gosubs_['grove_4_digit'] = grove_4_digit_gosub;
  var pin = generators.getPinName(block);
  var pin2 = getPin2(pin);
  var value = generators.valueToCode(block, 'VALUE', generators.ORDER_ATOMIC);
  var bgt = block.getFieldValue('BRIGHTNESS');
  var code = "\tgrove_4digit_brightness = " + bgt + " - 1 Max 8\n";
  code += "\tgrove_4digit_value = " + value + "\n";
  code += "\tgrove_4digit_scl = " + pin +"\n";
  code += "\tgrove_4digit_sda = " + pin2 + "\n";
  code += "\tgosub grove_4digit_updateDisplay\n";
  return generators.wrapOutput(block, code);
};

  var grove_bar_header = 'symbol grove_bar_brightness = grove_w1\n';
  grove_bar_header += 'symbol grove_bar_value = grove_w2\n';
  grove_bar_header += 'symbol grove_bar_scl = grove_w3\n';
  grove_bar_header += 'symbol grove_bar_sda = grove_w4\n';
  grove_bar_header += 'symbol grove_bar_mode = grove_w7\n';

  var grove_bar_gosub = 'grove_bar_updateDisplay:\n';
  grove_bar_gosub += '\tlow grove_bar_sda\n';
  grove_bar_gosub += '\tlow grove_bar_scl\n';
  grove_bar_gosub += '\tlookup grove_bar_brightness, ($01,$03,$07,$0F,$1F,$3F,$7F,$FF,$00),grove_bar_brightness\n';
  
  grove_bar_gosub += '\tgrove_w5 = $0000 : gosub grove_bar_sendWord\n';
  grove_bar_gosub += '\tgrove_w5 = 10: gosub grove_bar_sendLed\n';
  grove_bar_gosub += '\tgrove_w5 = 9: gosub grove_bar_sendLed\n';
  grove_bar_gosub += '\tgrove_w5 = 8: gosub grove_bar_sendLed\n';
  grove_bar_gosub += '\tgrove_w5 = 7: gosub grove_bar_sendLed\n';
  grove_bar_gosub += '\tgrove_w5 = 6: gosub grove_bar_sendLed\n';
  grove_bar_gosub += '\tgrove_w5 = 5: gosub grove_bar_sendLed\n';
  grove_bar_gosub += '\tgrove_w5 = 4: gosub grove_bar_sendLed\n';
  grove_bar_gosub += '\tgrove_w5 = 3: gosub grove_bar_sendLed\n';
  grove_bar_gosub += '\tgrove_w5 = 2: gosub grove_bar_sendLed\n';
  grove_bar_gosub += '\tgrove_w5 = 1: gosub grove_bar_sendLed\n';
  grove_bar_gosub += '\tgosub grove_bar_sendWord\n';
  grove_bar_gosub += '\tgosub grove_bar_sendWord\n';

  grove_bar_gosub += '\tlow grove_bar_sda\n';
  grove_bar_gosub += '\thigh grove_bar_sda : low grove_bar_sda\n';
  grove_bar_gosub += '\thigh grove_bar_sda : low grove_bar_sda\n';
  grove_bar_gosub += '\thigh grove_bar_sda : low grove_bar_sda\n';
  grove_bar_gosub += '\thigh grove_bar_sda : low grove_bar_sda\n';
  grove_bar_gosub += '\treturn\n\n';

  grove_bar_gosub += 'grove_bar_sendLed:\n';
  grove_bar_gosub += '\tif grove_bar_mode = 0x20 then\n';
  grove_bar_gosub += '\t\tif grove_bar_value <> grove_w5 then\n';
  grove_bar_gosub += '\t\t\tgrove_w5 = 0\n';
  grove_bar_gosub += '\t\tend if\n';
  grove_bar_gosub += '\telse if grove_bar_mode = 0x10 then\n';
  grove_bar_gosub += '\t\tif grove_bar_value < grove_w5 then\n';
  grove_bar_gosub += '\t\t\tgrove_w5 = 0 \n';
  grove_bar_gosub += '\t\tend if\n';
  grove_bar_gosub += '\telse\n';
  grove_bar_gosub += '\t\tgrove_w6 = grove_bar_value + grove_w5\n';
  grove_bar_gosub += '\t\tif grove_w6 < 11 then\n';
  grove_bar_gosub += '\t\t\tgrove_w5 = 0\n';
  grove_bar_gosub += '\t\tend if\n';
  grove_bar_gosub += '\tend if\n';
  grove_bar_gosub += '\tif grove_w5 <> 0 then\n';
  grove_bar_gosub += '\t\tgrove_w5 = grove_bar_brightness\n';
  grove_bar_gosub += '\tend if\n';
  grove_bar_gosub += 'grove_bar_sendWord:\n';
  grove_bar_gosub += '\tfor grove_w6 = 0 to 15\n';
  grove_bar_gosub += '\t\tif grove_w5 >= $8000 then\n';
  grove_bar_gosub += '\t\t\thigh grove_bar_sda\n';
  grove_bar_gosub += '\t\telse\n';
  grove_bar_gosub += '\t\t\tlow grove_bar_sda\n';
  grove_bar_gosub += '\t\tend if\n';
  grove_bar_gosub += '\t\ttoggle grove_bar_scl\n';
  grove_bar_gosub += '\t\tgrove_w5 = grove_w5 * 2\n';
  grove_bar_gosub += '\tnext grove_w6 \n';
  grove_bar_gosub += '\treturn\n\n';

  var grove_bar_gosub1 = 'grove_bar_updateCircular:\n';
  grove_bar_gosub1 += '\tlow grove_bar_sda\n';
  grove_bar_gosub1 += '\tlow grove_bar_scl\n';
  grove_bar_gosub1 += '\tlookup grove_bar_brightness, ($01,$03,$07,$0F,$1F,$3F,$7F,$FF,$00),grove_bar_brightness\n';
  
  grove_bar_gosub1 += '\tgrove_w5 = $0000 : gosub grove_bar_sendWord1\n';
  grove_bar_gosub1 += '\tgrove_w5 = 24: gosub grove_bar_sendLed1\n';
  grove_bar_gosub1 += '\tgrove_w5 = 23: gosub grove_bar_sendLed1\n';
  grove_bar_gosub1 += '\tgrove_w5 = 22: gosub grove_bar_sendLed1\n';
  grove_bar_gosub1 += '\tgrove_w5 = 21: gosub grove_bar_sendLed1\n';
  grove_bar_gosub1 += '\tgrove_w5 = 20: gosub grove_bar_sendLed1\n';
  grove_bar_gosub1 += '\tgrove_w5 = 19: gosub grove_bar_sendLed1\n';
  grove_bar_gosub1 += '\tgrove_w5 = 18: gosub grove_bar_sendLed1\n';
  grove_bar_gosub1 += '\tgrove_w5 = 17: gosub grove_bar_sendLed1\n';
  grove_bar_gosub1 += '\tgrove_w5 = 16: gosub grove_bar_sendLed1\n';
  grove_bar_gosub1 += '\tgrove_w5 = 15: gosub grove_bar_sendLed1\n';
  grove_bar_gosub1 += '\tgrove_w5 = 14: gosub grove_bar_sendLed1\n';
  grove_bar_gosub1 += '\tgrove_w5 = 13: gosub grove_bar_sendLed1\n';
  grove_bar_gosub1 += '\tgrove_w5 = 12: gosub grove_bar_sendLed1\n';
  grove_bar_gosub1 += '\tgrove_w5 = 11: gosub grove_bar_sendLed1\n';
  grove_bar_gosub1 += '\tgrove_w5 = 10: gosub grove_bar_sendLed1\n';
  grove_bar_gosub1 += '\tgrove_w5 = 9: gosub grove_bar_sendLed1\n';
  grove_bar_gosub1 += '\tgrove_w5 = 8: gosub grove_bar_sendLed1\n';
  grove_bar_gosub1 += '\tgrove_w5 = 7: gosub grove_bar_sendLed1\n';
  grove_bar_gosub1 += '\tgrove_w5 = 6: gosub grove_bar_sendLed1\n';
  grove_bar_gosub1 += '\tgrove_w5 = 5: gosub grove_bar_sendLed1\n';
  grove_bar_gosub1 += '\tgrove_w5 = 4: gosub grove_bar_sendLed1\n';
  grove_bar_gosub1 += '\tgrove_w5 = 3: gosub grove_bar_sendLed1\n';
  grove_bar_gosub1 += '\tgrove_w5 = 2: gosub grove_bar_sendLed1\n';
  grove_bar_gosub1 += '\tgrove_w5 = 1: gosub grove_bar_sendLed1\n';

  grove_bar_gosub1 += '\tlow grove_bar_sda\n';
  grove_bar_gosub1 += '\thigh grove_bar_sda : low grove_bar_sda\n';
  grove_bar_gosub1 += '\thigh grove_bar_sda : low grove_bar_sda\n';
  grove_bar_gosub1 += '\thigh grove_bar_sda : low grove_bar_sda\n';
  grove_bar_gosub1 += '\thigh grove_bar_sda : low grove_bar_sda\n';
  grove_bar_gosub1 += '\treturn\n\n';
  
  grove_bar_gosub1 += 'grove_bar_sendLed1:\n';
  grove_bar_gosub1 += '\tif grove_bar_mode = 0x20 then\n';
  grove_bar_gosub1 += '\t\tif grove_bar_value <> grove_w5 then\n';
  grove_bar_gosub1 += '\t\t\tgrove_w5 = 0\n';
  grove_bar_gosub1 += '\t\tend if\n';
  grove_bar_gosub1 += '\telse if grove_bar_mode = 0x10 then\n';
  grove_bar_gosub1 += '\t\tif grove_bar_value < grove_w5 then\n';
  grove_bar_gosub1 += '\t\t\tgrove_w5 = 0 \n';
  grove_bar_gosub1 += '\t\tend if\n';
  grove_bar_gosub1 += '\telse\n';
  grove_bar_gosub1 += '\t\tgrove_w6 = grove_bar_value + grove_w5\n';
  grove_bar_gosub1 += '\t\tif grove_w6 < 21 then\n';
  grove_bar_gosub1 += '\t\t\tgrove_w5 = 0\n';
  grove_bar_gosub1 += '\t\tend if\n';
  grove_bar_gosub1 += '\tend if\n';
  grove_bar_gosub1 += '\tif grove_w5 <> 0 then\n';
  grove_bar_gosub1 += '\t\tgrove_w5 = grove_bar_brightness\n';
  grove_bar_gosub1 += '\tend if\n';
  grove_bar_gosub1 += 'grove_bar_sendWord1:\n';
  grove_bar_gosub1 += '\tfor grove_w6 = 0 to 15\n';
  grove_bar_gosub1 += '\t\tif grove_w5 >= $8000 then\n';
  grove_bar_gosub1 += '\t\t\thigh grove_bar_sda\n';
  grove_bar_gosub1 += '\t\telse\n';
  grove_bar_gosub1 += '\t\t\tlow grove_bar_sda\n';
  grove_bar_gosub1 += '\t\tend if\n';
  grove_bar_gosub1 += '\t\ttoggle grove_bar_scl\n';
  grove_bar_gosub1 += '\t\tgrove_w5 = grove_w5 * 2\n';
  grove_bar_gosub1 += '\tnext grove_w6 \n';
  grove_bar_gosub1 += '\treturn\n\n';
  
generators['grove_led_bar'] = function(block) {
  if (!generators.definitions_['grove_symbol_header']) generators.definitions_['grove_symbol_header'] = grove_symbol_header;
  if (!generators.definitions_['grove_led_bar']) generators.definitions_['grove_led_bar'] = grove_bar_header;
  if (!generators.gosubs_['grove_led_bar']) generators.gosubs_['grove_led_bar'] = grove_bar_gosub;
  var pin = generators.getPinName(block);
  var pin2 = getPin2(pin);
  var value = generators.valueToCode(block, 'VALUE', generators.ORDER_ATOMIC);
  var bgt = block.getFieldValue('BRIGHTNESS');
  var mode = block.getFieldValue('MODE');
  var code = "\tgrove_bar_brightness = " + bgt + "\n";
  code += "\tgrove_bar_mode = " + mode + "\n";
  code += "\tgrove_bar_value = " + value + " max 10\n";
  code += "\tgrove_bar_sda = " + pin +"\n";
  code += "\tgrove_bar_scl = " + pin2 + "\n";
  code += "\tgosub grove_bar_updateDisplay\n";
  return generators.wrapOutput(block, code);
};

generators['grove_circular_led'] = function(block) {
  if (!generators.definitions_['grove_symbol_header']) generators.definitions_['grove_symbol_header'] = grove_symbol_header;
  if (!generators.definitions_['grove_led_bar']) generators.definitions_['grove_led_bar'] = grove_bar_header;
  if (!generators.gosubs_['grove_led_bar1']) generators.gosubs_['grove_led_bar1'] = grove_bar_gosub1;
  var pin = generators.getPinName(block);
  var pin2 = getPin2(pin);
  var value = generators.valueToCode(block, 'VALUE', generators.ORDER_ATOMIC);
  var bgt = block.getFieldValue('BRIGHTNESS');
  var mode = block.getFieldValue('MODE');
  var code = "\tgrove_bar_brightness = " + bgt + "\n";
  code += "\tgrove_bar_mode = " + mode + "\n";
  code += "\tgrove_bar_value = " + value + " max 10\n";
  code += "\tgrove_bar_sda = " + pin +"\n";
  code += "\tgrove_bar_scl = " + pin2 + "\n";
  code += "\tgosub grove_bar_updateCircular\n";
  return generators.wrapOutput(block, code);
};

generators['grove_bluetooth_receive'] = function (block) {
  if (!generators.definitions_['grove_define_x2part']) generators.definitions_['grove_define_x2part'] = x2part_header;
  if (!generators.definitions_['grove_define_uart_9600']) generators.definitions_['grove_define_uart_9600'] = uart_9600_header;
  var name = generators.variableDB_.getName(block.getFieldValue("NAME"), BlocklyLib.Variables.NAME_TYPE);
  var code = "#ifdef _X2part\n\tif hserflag = 1 then\n\t\tdo\n\t\t\tlet "
  code += name + " = @ptrinc\n\t\tloop while ptr <> hserptr\n\t\tlet hserflag = 0\n\telse\n\t\tlet "
  code += name + " = 0\n\tend if\n";
  code += "#else\n\thserin " + name + "\n#endif\n";
  return generators.wrapOutput(block, code);
};

generators['grove_bluetooth_send'] = function (block) {
  if (!generators.definitions_['grove_define_x2part']) generators.definitions_['grove_define_x2part'] = x2part_header;    
  if (!generators.definitions_['grove_define_uart_9600']) generators.definitions_['grove_define_uart_9600'] = uart_9600_header;
  var value = generators.valueToCode(block, 'VALUE', generators.ORDER_ATOMIC);
  if (!value) value = 0;
  var code = "\thserout 0, ( " + value + " )\n";
  code += "\tpause 10\n";
  return generators.wrapOutput(block, code);
};

  var grove_mp3_header = '\t;Initialise MP3 module\n';
  grove_mp3_header += "\thserout 0, ($7E, $FF, $06, $09, $00, $00, $02, $EF)\n";
  
 generators['grove_mp3_v2'] = function(block) {
  if (!generators.definitions_['grove_define_x2part']) generators.definitions_['grove_define_x2part'] = x2part_header;    
  if (!generators.definitions_['grove_define_uart_9600']) generators.definitions_['grove_define_uart_9600'] = uart_9600_header;
  if (!generators.definitions_['grove_define_mp3']) generators.definitions_['grove_define_mp3'] = grove_mp3_header;  
  var value = generators.valueToCode(block, 'VALUE', generators.ORDER_ATOMIC);
  var cmd = block.getFieldValue('COMMAND');
  var  code = "";
  code += "\thserout 0, ($7E, $FF, $06, ";
  if (cmd=="play") {
    code += "$12, $00, $00, " + value + ", $EF)\t;play\n";
  }
  else if (cmd=="stop") {
    code += "$16, $00, $00, $00, $EF)\t;stop\n";
  }
  else if (cmd=="pause") {
    code += "$0E, $00, $00, $00, $EF)\t;pause\n";
  }
  else if (cmd=="resume") {
    code += "$0D, $00, $00, $00, $EF)\t;resume\n";
  }
  else if (cmd=="volume") {
    code += "$06, $00, $00, " + value + ", $EF)\t;volume\n";
  }
  code += "\tpause 100\n";
  return generators.wrapOutput(block, code);
}; 

  var i2c_motor_header = '\t;Initialise I2C motor module\n';
  i2c_motor_header += '\thi2csetup I2CMASTER, 0x1E, I2CSLOW, I2CBYTE\n';
  i2c_motor_header += '\thi2cout 0x84, (0x02, 0x01)\n';
  
generators['grove_i2c_motor'] = function(block) {
  var type = block.getFieldValue('TYPE');
  var dir1 = block.getFieldValue('DIRECTION1');
  var dir2 = block.getFieldValue('DIRECTION2');
  var speed1 = generators.valueToCode(block, 'SPEED1', generators.ORDER_ATOMIC);
  var speed2 = generators.valueToCode(block, 'SPEED2', generators.ORDER_ATOMIC);
  var code = "";

  if (type == "mini") {
      if (!speed1) speed1 = 63;
      if (!speed2) speed2 = 63;
      if (speed1 > 63) speed1 = 63;
      if (speed2 > 63) speed2 = 63;
      speed1 = speed1 << 2;
      speed2 = speed2 << 2;
      code +=    "\thi2csetup I2CMASTER, 0xC4, I2CSLOW, I2CBYTE\n";
      code +=    "\thi2cout 0x01, (0x80) ; clear any faults\n";
      if (dir1=="F") {
        speed1 |= 0x02;  
      }
      else if (dir1=="B") {
        speed1 |= 0x01;  
      }
      else if (dir1=="S") {
        speed1 = 0x00;  
      }
      else if (dir1=="K") {
        speed1 = 0x03;  
      }
      code +=    "\thi2cout 0x00, ("+speed1+")\n";
      code +=    "\thi2csetup I2CMASTER, 0xC0, I2CSLOW, I2CBYTE\n";
      code +=    "\thi2cout 0x01, (0x80) ; clear any faults\n";
      if (dir2=="F") {
        speed2 |= 0x02;  
      }
      else if (dir2=="B") {
        speed2 |= 0x01;  
      }
      else if (dir2=="S") {
        speed2 = 0x00;
      }
      else if (dir2=="K") {
        speed2 = 0x03;
      }
      code +=    "\thi2cout 0x00, ("+speed2+")\n";
    } else { //for any other type than mini, type is the slave addess
      if (!generators.definitions_['grove_i2c_motor'+type]) generators.definitions_['grove_i2c_motor'+type] = i2c_motor_header.replace("0x1E",type);
      code += "\thi2csetup I2CMASTER, " + type + ", I2CSLOW, I2CBYTE\n";
      if (!speed1) speed1 = 255;
      if (!speed2) speed2 = 255;
     
      if (dir1=="S") {
        dir1 = "F";
        speed1 = 0x00;  
      }
      else if (dir1=="K") {
        dir1 = "F";
        speed1 = 0x00;  
      }
      if (dir2=="S") {
        dir2 = "F";
        speed2 = 0x00;
      }
      else if (dir2=="K") {
        dir2 = "F";
        speed2 = 0x00;
      }
      if (dir1 == "F" && dir2 == "F") code += "\thi2cout 0xAA, (0x0a, 0x01)\n"; // 1CW 2CW - Both clockwise
      if (dir1 == "B" && dir2 == "B") code += "\thi2cout 0xAA, (0x05, 0x01)\n"; // 1AC 2AC - Both anti-clockwise
      if (dir1 == "B" && dir2 == "F") code += "\thi2cout 0xAA, (0x09, 0x01)\n"; // 1AC 2CW
      if (dir1 == "F" && dir2 == "B") code += "\thi2cout 0xAA, (0x06, 0x01)\n"; // 1CW 2AC      
      code += "\thi2cout 0x82, ("+speed1+", "+speed2+")\n";
  }
  return generators.wrapOutput(block, code);
};
  
});
